!                                                                                                                                   
!                                              _____ _            _ ______ _                                                        
!                                             / ____| |          | |  ____| |                                                       
!                                            | (___ | |_ ___  ___| | |__  | | _____      __                                         
!                                             \___ \| __/ _ \/ _ \ |  __| | |/ _ \ \ /\ / /                                         
!                                             ____) | ||  __/  __/ | |    | | (_) \ V  V /                                          
!                                            |_____/ \__\___|\___|_|_|    |_|\___/ \_/\_/                                           
!                                                                                                                                   
!*******************************                   Copyright Dr Chris Hulme-Smith                    *******************************
!*******************************                       University of Cambridge                       *******************************
!*******************************                            September 2016                           *******************************
!**********************************************************************************************************************************!
PROGRAM MECHMOD
IMPLICIT NONE
!**********************************************************************************************************************************!


!**********************************************************************************************************************************!
INTEGER::ELENUM,CNO,PPTPOPS
INTEGER,DIMENSION(2)::WHATUNIT
INTEGER,ALLOCATABLE,DIMENSION(:)::ATOMNO,YOUNGUNIT,PPTSHAPE,RELEQM
INTEGER,DIMENSION(118)::RECORDED_YOUNGUNIT
INTEGER,PARAMETER::SETUNIT=13,COMPUNIT=14,MASSUNIT=15,COEFUNIT=16,PPTUNIT=17
!----------------------------------------------------------------------------------------------------------------------------------!
DOUBLE PRECISION::GRAINSIZE,SIZESTRENGTH,REFINEMENT,PEIERLS,YOUNGMOD,BULKMOD,POISSON,MATRIXSPACING,SHEARMOD,MATRIXATOMVOL,AMBIENT
DOUBLE PRECISION::FLOW_TEMP_AIM,STR_RATE_AIM,ACTIVATION,EXPONENT,STR_RATE_STATIC,TAYLOR,STRAINSTEP,PATHCONST,ENDSTRAIN,HARDCONST
DOUBLE PRECISION::RECOVERYCONST,GB_ENERGY,ZENER_FACTOR,SHEARTEMP,DEFACT,REXACT,REXCONST,REXRHO,GROWTHACT
DOUBLE PRECISION,ALLOCATABLE,DIMENSION(:,:)::BULKCOMP,SOLCOMP,SOLCON,SOLSTRENGTH
DOUBLE PRECISION,ALLOCATABLE,DIMENSION(:)::COHERCON,MASSES,PPTDENSITY,PPTSPACING,PPTFACTOR,PPTSIZE,PPTFRAC,DELTA,EPSILON
DOUBLE PRECISION,ALLOCATABLE,DIMENSION(:)::OROWANSTRESS,CUTSTRESS,PPTBETA,PPTDELTA,PPTEPSILON,PPTKAPPA,PPTBULK,PPTATOMVOL
DOUBLE PRECISION,DIMENSION(118)::RECORDED_MASSES
DOUBLE PRECISION,DIMENSION(0:3,118)::RECORDED_COEFFS
DOUBLE PRECISION,DIMENSION(3)::TOTSOLSTR,YIELD,FLOW
DOUBLE PRECISION,DIMENSION(2)::MOBILITY
!----------------------------------------------------------------------------------------------------------------------------------!
CHARACTER(LEN=18),PARAMETER::SETFILE='config_mechmod.dat'
CHARACTER(LEN=60)::COMPFILE,PPTFILE,RESULTSFILE
CHARACTER(LEN=17)::MASSFILE
CHARACTER(LEN=18)::COEFILE
CHARACTER(LEN=2),ALLOCATABLE,DIMENSION(:)::ELEMENTS
CHARACTER(LEN=10),ALLOCATABLE,DIMENSION(:)::PPTNAME
CHARACTER(LEN=2),DIMENSION(118)::RECORDED_ELEMENTS
CHARACTER(LEN=1)::FERTYPE
CHARACTER(LEN=6),DIMENSION(3)::YIELDMODELNAME
CHARACTER(LEN=8)::DATE
CHARACTER(LEN=10)::NOWTIME
!**********************************************************************************************************************************!
! In order to pass unallocated arrays to a subroutine, that subroutine must be declared inside an "explicit interface", i.e. the 
! subroutine ... end subroutine must appear once wrapped inside interface ... end interface and this must be before "END PROGRAM".
! The full subroutine is in its expected location after the end of the program code.
!**********************************************************************************************************************************!
INTERFACE
!**********************************************************************************************************************************!
      SUBROUTINE ALLOCATE_THEM(ELENUM,BULKCOMP,SOLCOMP,MASSES,ATOMNO,ELEMENTS,SOLCON,SOLSTRENGTH,YOUNGUNIT,PPTPOPS,PPTSIZE,&
     & PPTDENSITY,PPTSPACING,PPTSHAPE,PPTFACTOR,COHERCON,PPTFRAC,PPTNAME,DELTA,EPSILON,RELEQM,OROWANSTRESS,CUTSTRESS,PPTBETA,&
     & PPTDELTA,PPTEPSILON,PPTKAPPA,PPTBULK,PPTATOMVOL)
!----------------------------------------------------------------------------------------------------------------------------------!
            INTEGER,INTENT(IN)::ELENUM,PPTPOPS
            INTEGER,ALLOCATABLE,DIMENSION(:)::ATOMNO,YOUNGUNIT,PPTSHAPE,RELEQM
            DOUBLE PRECISION,ALLOCATABLE,DIMENSION(:,:)::BULKCOMP,SOLCOMP,SOLCON,SOLSTRENGTH
            DOUBLE PRECISION,ALLOCATABLE,DIMENSION(:)::MASSES,PPTFACTOR,PPTSIZE,PPTSPACING,PPTDENSITY,PPTFRAC,DELTA,EPSILON,COHERCON
            DOUBLE PRECISION,ALLOCATABLE,DIMENSION(:)::OROWANSTRESS,CUTSTRESS,PPTBETA,PPTDELTA,PPTEPSILON,PPTKAPPA,PPTBULK
            DOUBLE PRECISION,ALLOCATABLE,DIMENSION(:)::PPTATOMVOL
            CHARACTER(LEN=2),ALLOCATABLE,DIMENSION(:)::ELEMENTS
            CHARACTER(LEN=10),ALLOCATABLE,DIMENSION(:)::PPTNAME
!----------------------------------------------------------------------------------------------------------------------------------!
      END SUBROUTINE ALLOCATE_THEM
!**********************************************************************************************************************************!
END INTERFACE
!**********************************************************************************************************************************!
!**********************************************************************************************************************************!
! this is the main program.  All the below subroutines and functions are called (directly or indirectly) from this section of code.
!----------------------------------------------------------------------------------------------------------------------------------!
! initialise (give arbitrary values to) all variables that are used without otherwise havIng values set to them.. Failure to do this
! can cause instabilities, depending on the processor and compiler used.
CALL LOGO()

! initialise variables.
CALL INITIALISATION(ELENUM,COMPFILE,PPTFILE,WHATUNIT,RECORDED_ELEMENTS,RECORDED_MASSES,RECORDED_COEFFS,COEFILE,TOTSOLSTR,FERTYPE,&
    & GRAINSIZE,SIZESTRENGTH,PPTPOPS,BULKMOD,YOUNGMOD,POISSON,MATRIXSPACING,YIELDMODELNAME,STRAINSTEP,PATHCONST,ENDSTRAIN,&
    & HARDCONST,RECOVERYCONST)

! read in the settings under which the program will run, including the names of the files containing the compositions.
CALL READ_SETTINGS(SETUNIT,SETFILE,ELENUM,COMPFILE,PPTFILE,WHATUNIT,MASSFILE,COEFILE,FERTYPE,GRAINSIZE,BULKMOD,YOUNGMOD,POISSON,&
     & PPTPOPS,MATRIXSPACING,SHEARMOD,TAYLOR,MATRIXATOMVOL,FLOW_TEMP_AIM,STR_RATE_AIM,ACTIVATION,EXPONENT,STR_RATE_STATIC,AMBIENT,&
     & STRAINSTEP,PATHCONST,ENDSTRAIN,HARDCONST,RECOVERYCONST,GB_ENERGY,MOBILITY,SHEARTEMP,PEIERLS,DEFACT,REXACT,REXCONST,REXRHO,&
     & GROWTHACT,RESULTSFILE)
PRINT 001, PEIERLS

! set the dimension of variable-sized ("allocatable" in FORTRAN parlance) arrays and initialise them.
CALL ALLOCATE_THEM(ELENUM,BULKCOMP,SOLCOMP,MASSES,ATOMNO,ELEMENTS,SOLCON,SOLSTRENGTH,YOUNGUNIT,PPTPOPS,PPTSIZE,PPTDENSITY,&
     & PPTSPACING,PPTSHAPE,PPTFACTOR,COHERCON,PPTFRAC,PPTNAME,DELTA,EPSILON,RELEQM,OROWANSTRESS,CUTSTRESS,PPTBETA,PPTDELTA,&
     & PPTEPSILON,PPTKAPPA,PPTBULK,PPTATOMVOL)

! read in information relating to precipitate populations (if any).
CALL READ_PPTS(PPTUNIT,PPTFILE,PPTPOPS,PPTSIZE,PPTDENSITY,PPTFRAC,PPTSPACING,PPTSHAPE,PPTFACTOR,PPTNAME,RELEQM,PPTBETA,PPTDELTA,&
     & PPTEPSILON,PPTKAPPA,PPTBULK,PPTATOMVOL)

! calculate the effect of grain refinement strengthening.
SIZESTRENGTH = REFINEMENT(GRAINSIZE)
PRINT 002, SIZESTRENGTH

! then read in both the bulk composition and the composition of the matrix.
CALL READ_COMP(COMPUNIT,COMPFILE,ELENUM,ELEMENTS,BULKCOMP,SOLCOMP,WHATUNIT)

! find the atomic masses of the elements used.
CALL FIND_MASS(ELENUM,ELEMENTS,RECORDED_ELEMENTS,MASSES,MASSUNIT,MASSFILE)

! calculate the composition in whatever units have not been used to input the composition.
! for both the bulk composition and the solid solution matrix composition, calculate the composition in whichever three of atomic
! percent, mass percent, molar fraction and mass fraction were not used to specify the composition.
CALL CALC_COMPS(ELENUM,ELEMENTS,MASSES,WHATUNIT,BULKCOMP,SOLCOMP)

! read in the coefficients of the expressions for solid solution strengthening (one set each for the expressions of Young et al.,
! Lacy and Leslie, as documented in the report).
CALL READ_COEFFS(COEFILE,COEFUNIT,RECORDED_COEFFS,RECORDED_YOUNGUNIT)

! extract the coefficients for each expression for all substitutional elements specified in the composition.
CALL FIND_COEFFS(ELENUM,ELEMENTS,RECORDED_ELEMENTS,RECORDED_COEFFS,SOLCON,ATOMNO,FERTYPE,CNO,RECORDED_YOUNGUNIT,YOUNGUNIT)
! for some reason, the value of PPTSIZE(1) is set to zero around this point in the code, so copy the value to another variable and
! then write it back.
CALL COHERENCY(PPTPOPS,PPTSIZE,PPTDENSITY,PPTSPACING,PPTSHAPE,PPTFACTOR,COHERCON,PPTFRAC,DELTA,EPSILON,YOUNGMOD,BULKMOD,&
     & POISSON,MATRIXSPACING,RELEQM,PPTNAME)

CALL OROWAN(PPTPOPS,PPTSIZE,PPTDENSITY,PPTNAME,MATRIXSPACING,POISSON,SHEARMOD,TAYLOR,OROWANSTRESS)

CALL CUTTING(PPTPOPS,PPTFRAC,PPTSIZE,TAYLOR,CUTSTRESS,SHEARMOD,PPTBULK,MATRIXSPACING,PPTBETA,PPTDELTA,&
     & PPTEPSILON,PPTKAPPA,MATRIXATOMVOL,PPTATOMVOL,YOUNGMOD,POISSON,PPTNAME)

! for some reason, this subroutine messes up all the arrays containing precipitate information, so this subroutine must either be
! after the precipitate subroutines or the data must be read in after SOLSOLSTR.  For simplicity, I have chosen the former solution.
CALL SOLSOLSTR(ELENUM,ELEMENTS,SOLCON,SOLCOMP,SOLSTRENGTH,TOTSOLSTR,FERTYPE,CNO,YOUNGUNIT)
!PRINT *, 'CHECKPOINT XI'
! add all the contributions from those sources that do not depend on strain (at least not in this model).
CALL ADDITALL(PPTPOPS,CUTSTRESS,OROWANSTRESS,COHERCON,PPTNAME,PEIERLS,SIZESTRENGTH,TOTSOLSTR,YIELDMODELNAME,YIELD)

! do the conversion to flow stress.
CALL YIELDTOFLOW(YIELD,FLOW,FLOW_TEMP_AIM,STR_RATE_AIM,ACTIVATION,FERTYPE,EXPONENT,STR_RATE_STATIC,AMBIENT,ZENER_FACTOR)
! account for work hardening and save the strain-dependent work hardening contribution and flow stresses
CALL WORKHARDENING(SHEARMOD,FERTYPE,GRAINSIZE,MATRIXSPACING,STRAINSTEP,PATHCONST,ENDSTRAIN,TAYLOR,HARDCONST,RECOVERYCONST,&
    & STR_RATE_AIM,FLOW_TEMP_AIM,SHEARTEMP,GB_ENERGY,MOBILITY,ZENER_FACTOR,YIELD,RESULTSFILE,DEFACT,REXACT,REXCONST,REXRHO,&
    & GROWTHACT)

! make a closing remark and print the logo again.
PRINT 003
CALL DATE_AND_TIME(DATE,NOWTIME)
PRINT 004, DATE(7:8),DATE(5:6),DATE(1:4),NOWTIME(1:2),NOWTIME(3:4),NOWTIME(5:6)
PRINT *, ''
CALL LOGO()
!**********************************************************************************************************************************!
 001  FORMAT('The Peierls (intrinsic) strength is set to ',F5.1,' MPa.')
 002  FORMAT('The contribution to yield strength due to grain refinement is ',F6.1,' MPa.')
 003  FORMAT('All contributions to yield stress successfully modelled.  CaNDYFloSS will save all data and close all files.  ',&
    & 'Goodbye!')
 004  FORMAT('CaNDYFloSS completed and terminated at ',A2,'/',A2,'/',A4,' at ',A2,':',A2,':',A2)
!**********************************************************************************************************************************!
END PROGRAM MECHMOD
!**********************************************************************************************************************************!







!**********************************************************************************************************************************!
SUBROUTINE LOGO()
IMPLICIT NONE
!----------------------------------------------------------------------------------------------------------------------------------!
PRINT 001
PRINT 002
PRINT 003
PRINT 004
PRINT 005
PRINT 006
PRINT *, ''
PRINT 007
PRINT 008
PRINT 009
PRINT 010
PRINT 007
PRINT *, ''
!----------------------------------------------------------------------------------------------------------------------------------!

001  FORMAT('                    _____ _            _ ______ _                               ')
002  FORMAT('                   / ____| |          | |  ____| |                              ')
003  FORMAT('                  | (___ | |_ ___  ___| | |__  | | _____      __                ')
004  FORMAT('                   \___ \| __/ _ \/ _ \ |  __| | |/ _ \ \ /\ / /                ')
005  FORMAT('                   ____) | ||  __/  __/ | |    | | (_) \ V  V /                 ')
006  FORMAT('                  |_____/ \__\___|\___|_|_|    |_|\___/ \_/\_/                  ')
007  FORMAT(80('-'))
008  FORMAT('                        Copyright Dr Chris Hulme-Smith                          ')
009  FORMAT('                            University of Cambridge                             ')
010  FORMAT('                                 September 2017                                 ')
!----------------------------------------------------------------------------------------------------------------------------------!
END SUBROUTINE LOGO
!**********************************************************************************************************************************!


!**********************************************************************************************************************************!
SUBROUTINE INITIALISATION(ELENUM,COMPFILE,PPTFILE,WHATUNIT,RECORDED_ELEMENTS,RECORDED_MASSES,RECORDED_COEFFS,COEFILE,TOTSOLSTR,&
    & FERTYPE,GRAINSIZE,SIZESTRENGTH,PPTPOPS,BULKMOD,YOUNGMOD,POISSON,MATRIXSPACING,YIELDMODELNAME,STRAINSTEP,PATHCONST,ENDSTRAIN,&
    & HARDCONST,RECOVERYCONST)
IMPLICIT NONE
!----------------------------------------------------------------------------------------------------------------------------------!
INTEGER::I0,J0
INTEGER,INTENT(OUT)::ELENUM,PPTPOPS
INTEGER,DIMENSION(2),INTENT(OUT)::WHATUNIT

DOUBLE PRECISION,INTENT(OUT)::GRAINSIZE,SIZESTRENGTH,BULKMOD,YOUNGMOD,POISSON,MATRIXSPACING,STRAINSTEP,PATHCONST,ENDSTRAIN
DOUBLE PRECISION,INTENT(OUT)::HARDCONST,RECOVERYCONST
DOUBLE PRECISION,DIMENSION(118),INTENT(OUT)::RECORDED_MASSES
DOUBLE PRECISION,DIMENSION(0:3,118),INTENT(OUT)::RECORDED_COEFFS
DOUBLE PRECISION,DIMENSION(3),INTENT(OUT)::TOTSOLSTR

CHARACTER(LEN=1),INTENT(OUT)::FERTYPE
CHARACTER(LEN=20),INTENT(OUT)::COMPFILE,PPTFILE
CHARACTER(LEN=18),INTENT(OUT)::COEFILE
CHARACTER(LEN=2),DIMENSION(118),INTENT(OUT)::RECORDED_ELEMENTS
CHARACTER(LEN=6),DIMENSION(3),INTENT(OUT)::YIELDMODELNAME
!----------------------------------------------------------------------------------------------------------------------------------!
! initialise all variables that do not have values explicitly passed to them before being used. The values used are arbitrary and
! should not affect the program - the variables should have other values written to them before they are used for anything.
GRAINSIZE = -4.2330D-02
SIZESTRENGTH = -3.94992D+04
BULKMOD = -4.8956258D+03
YOUNGMOD = -1.75856D-08
POISSON = -6.4386574754D-05
MATRIXSPACING = -5.02927454D+04
ELENUM = 136
PPTPOPS = -45
WHATUNIT(1) = 43
WHATUNIT(2) = 56
COMPFILE = 'neuejmsdseoiaefljknf'
PPTFILE = 'idhjfnvmcksjusndhres'
COEFILE='rsjbsrgjrgknhrlncd'
INIT_RECORDED:DO I0 = 1,118,1
      RECORDED_MASSES(I0)=5.0654651D-01+DBLE(I0)
      RECORDED_ELEMENTS(I0)='sO'
      DO J0 = 0,3,1
            RECORDED_COEFFS(J0,I0) = 6.154198561D-01 + DBLE(I0) + 1.0D+02 * DBLE(J0)
            TOTSOLSTR(J0) = 2.563563D-1 + DBLE(J0)
      END DO
END DO INIT_RECORDED
YIELDMODELNAME(1) = 'Young '
YIELDMODELNAME(2) = 'Lacy  '
YIELDMODELNAME(3) = 'Leslie'
FERTYPE = 'Q'
STRAINSTEP = -0.2362486438742D-04
PATHCONST = -8.23782834634634D+03
ENDSTRAIN = -3.43785785378D+02
HARDCONST = -5.489347834784378D+09
RECOVERYCONST = -3.47262387247842D-02
!----------------------------------------------------------------------------------------------------------------------------------!
END SUBROUTINE INITIALISATION
!**********************************************************************************************************************************!



!**********************************************************************************************************************************!
SUBROUTINE READ_SETTINGS(SETUNIT,SETFILE,ELENUM,COMPFILE,PPTFILE,WHATUNIT,MASSFILE,COEFILE,FERTYPE,GRAINSIZE,BULKMOD,YOUNGMOD,&
     & POISSON,PPTPOPS,MATRIXSPACING,SHEARMOD,TAYLOR,MATRIXATOMVOL,FLOW_TEMP_AIM,STR_RATE_AIM,ACTIVATION,EXPONENT,&
     & STR_RATE_STATIC,AMBIENT,STRAINSTEP,PATHCONST,ENDSTRAIN,HARDCONST,RECOVERYCONST,GB_ENERGY,MOBILITY,PEIERLS,SHEARTEMP,&
     & DEFACT,REXACT,REXCONST,REXRHO,GROWTHACT,RESULTSFILE)
IMPLICIT NONE
!----------------------------------------------------------------------------------------------------------------------------------!
INTEGER::SETUNIT,HANDLE1
INTEGER,INTENT(OUT)::PPTPOPS,ELENUM
INTEGER,DIMENSION(2),INTENT(OUT)::WHATUNIT
!----------------------------------------------------------------------------------------------------------------------------------!
DOUBLE PRECISION,INTENT(OUT)::GRAINSIZE,BULKMOD,YOUNGMOD,POISSON,MATRIXSPACING,SHEARMOD,TAYLOR,MATRIXATOMVOL,FLOW_TEMP_AIM
DOUBLE PRECISION,INTENT(OUT)::STR_RATE_AIM,ACTIVATION,EXPONENT,STR_RATE_STATIC,AMBIENT,STRAINSTEP,PATHCONST,ENDSTRAIN,HARDCONST
DOUBLE PRECISION,INTENT(OUT)::RECOVERYCONST,GB_ENERGY,SHEARTEMP,PEIERLS,DEFACT,REXACT,REXCONST,REXRHO,GROWTHACT
DOUBLE PRECISION,DIMENSION(2),INTENT(OUT)::MOBILITY
!----------------------------------------------------------------------------------------------------------------------------------!
CHARACTER(LEN=1),INTENT(OUT)::FERTYPE
CHARACTER(LEN=18)::SETFILE
CHARACTER(LEN=18),INTENT(OUT)::COEFILE
CHARACTER(LEN=60),INTENT(OUT)::COMPFILE,PPTFILE,RESULTSFILE
CHARACTER(LEN=17),INTENT(OUT)::MASSFILE
!----------------------------------------------------------------------------------------------------------------------------------!
! initialise all variables that did not already have values passed to them. The values used are arbitrary and should not affect the
! program - the variables should have other values written to them before they are used for anything.
HANDLE1 = 34
!----------------------------------------------------------------------------------------------------------------------------------!
! open the file with the name specified in the SETFILE variable using memory unit SETUNIT.  The 'OLD' status requires that the file
! already exist - if this is not so, the program will fail.  IOSTAT='HANDLE' gives a variable that allows some end-of-file errors to
! be caught and diagnosed, e.g. if some data is missing (0=no errors; <0 = end of record (line) or file reached sooner than
! expected; >0 some other error reading the file, e.g. unrecognised data or format type, such as expecting a number and finding
! letters.).  ACTION='READ' means the file is treated as read-only and is never altered by the program.
OPEN(UNIT=SETUNIT,IOSTAT=HANDLE1,FILE=SETFILE,STATUS='OLD',ACTION='READ')
! read the name of the composition file
READ(UNIT=SETUNIT,FMT=*) COMPFILE
! readin the name of the file containing information about precipitate populations.
READ(UNIT=SETUNIT,FMT=*) PPTFILE
! read the number of elements entered in the compositions file.
READ(UNIT=SETUNIT,FMT=*) ELENUM
! read the composition is expressed in wt% or at.%
READ(UNIT=SETUNIT,FMT=*) WHATUNIT(1)
READ(UNIT=SETUNIT,FMT=*) WHATUNIT(2)
! read the file containing atomic masses.
READ(UNIT=SETUNIT,FMT=*) MASSFILE
! read the name of the file containing the solid solution strengthening coefficients.
READ(UNIT=SETUNIT,FMT=*) COEFILE
! read the type of ferrite, "M" for martensite, "B" for bainite, "F" for other types of Ferrite.
READ(UNIT=SETUNIT,FMT=*) FERTYPE
! read the effective grainsize of the modelled material.
READ(UNIT=SETUNIT,FMT=*) GRAINSIZE
! read the bulk modulus of the modelled material.
READ(UNIT=SETUNIT,FMT=*) BULKMOD
! read the Young's modulus of the modelled material.
READ(UNIT=SETUNIT,FMT=*) YOUNGMOD
! read the Poisson ratio of the modelled material.
READ(UNIT=SETUNIT,FMT=*) POISSON
! read in the shear modulus of the modelled material.
READ(UNIT=SETUNIT,FMT=*) SHEARMOD
! read in the Taylor factor to be used in Orowan bowing calculation.
READ(UNIT=SETUNIT,FMT=*) TAYLOR
! read the information regarding each precipitate population in turn.
READ(UNIT=SETUNIT,FMT=*) PPTPOPS
! read the effective interatomic spacing in the matrix.
READ(UNIT=SETUNIT,FMT=*) MATRIXSPACING
! read in the atomic volume of the matrix.
READ(UNIT=SETUNIT,FMT=*) MATRIXATOMVOL
! read in the desired temperature for calculation of flow stress.
READ(UNIT=SETUNIT,FMT=*) FLOW_TEMP_AIM
! read in the desired strain rate for calculation of flow stress.
READ(UNIT=SETUNIT,FMT=*) STR_RATE_AIM
! read in the activation energy of dislocation motion - a value of 0 will result in no temperature effects, a value < 0 will cause a
! default value to be used, based on the type of ferrite specified and the values in Zener and Holloman, J. App. Phys 15 (1944)
! p.23-43.
READ(UNIT=SETUNIT,FMT=*) ACTIVATION
! read in the activation energy of disloction motion - a value of 0 will result in no temperature or strain rate effects, a value
! < 0 will cause a default value of 400.0 to be used, from Zener and Holloman, J. App. Phys 15 (1944),
! p.23-43
READ(UNIT=SETUNIT,FMT=*) EXPONENT
IF(EXPONENT .LT. 0.0D+00) THEN
    EXPONENT = 1.25D-02
END IF
! specify the strain rate used in the determination of yield stress - a default value of 0.01 / s is used if the specified value is
! negative.

READ(UNIT=SETUNIT,FMT=*) STR_RATE_STATIC
! If a negative target strain rate is specified, the target strain rate is set to be equal to that used to measure the yield stress.
! This has the effect of removing all strain rate effects from the flow stress calculation.
IF(STR_RATE_STATIC .LT. 0.0D+00) THEN
    STR_RATE_STATIC = 1.0D-02
END IF
IF(STR_RATE_AIM .LE. 0.0D+00) THEN
    STR_RATE_AIM = STR_RATE_STATIC
END IF
! specify the temperature (Celsius) used in the determination of yield stress - a default value of 20C is used if the specified
! value is below absolute zero.  The value is immediately converted to Kelvin.
READ(UNIT=SETUNIT,FMT=*) AMBIENT
IF(AMBIENT .LE. -2.7315D+02) THEN
    AMBIENT = 2.9315D+02
ELSE
    AMBIENT = AMBIENT + 2.7315D+02
END IF

! read in the step to be used in work hardening calculations
READ(UNIT=SETUNIT,FMT=*) STRAINSTEP
! read in a constant that affects dislocation mean free path (k in Bouaziz & Guelton, Mater. Sci. Eng A 319-321 (2001) p. 246-249.
READ(UNIT=SETUNIT,FMT=*) PATHCONST
! read in the maximum strai nto be considered in work hardening model.
READ(UNIT=SETUNIT,FMT=*) ENDSTRAIN
! read in the constant of proportionality that governs how sensitive the flow stress is to dislocation density.
READ(UNIT=SETUNIT,FMT=*) HARDCONST
! read in a variable that affects the rate of recovery (f in Bouaziz & Guelton, Mater. Sci. Eng A 319-321 (2001) p. 246-249.
READ(UNIT=SETUNIT,FMT=*) RECOVERYCONST
! read in the grain boundary energy in J m^(-2)
READ(UNIT=SETUNIT,FMT=*) GB_ENERGY
! read in the pre-exponential factor to calculate the mobility of grain boundaries, for use in recrystallisation calculations.
READ(UNIT=SETUNIT,FMT=*) MOBILITY(1)
! read in the activation energy for the mobility of grain boundaries, for use in recrystallisation calculations.
READ(UNIT=SETUNIT,FMT=*) MOBILITY(2)
! read in the name of the file to which strain-dependent yield stresses will be written.
READ(UNIT=SETUNIT,FMT=*) RESULTSFILE
! read in the temperature at which shear transformations are expected in the alloy during cooling - not used for FERTYPE='F', but
! value still needed to allow proper execution of the program.
READ(UNIT=SETUNIT,FMT=*) SHEARTEMP
! read in the Peierls stress.
READ(UNIT=SETUNIT,FMT=*) PEIERLS
! read in the effective activation energy for deformation in Joules.
READ(UNIT=SETUNIT,FMT=*) DEFACT
! read in the effective activation energy for recrystallisation in Joules.
READ(UNIT=SETUNIT,FMT=*) REXACT
! read in the exponent to which the timestep must be raised to calculate the fraction of material that recrystallises in each
!timestep.
READ(UNIT=SETUNIT,FMT=*) REXCONST
! read in the dislocation density of recrystallised material in m^(-2).
READ(UNIT=SETUNIT,FMT=*) REXRHO
! read in the effective activation energy for grain growth.
READ(UNIT=SETUNIT,FMT=*) GROWTHACT
! close the settings file
CLOSE(UNIT=SETUNIT,IOSTAT=HANDLE1,STATUS='KEEP')
!----------------------------------------------------------------------------------------------------------------------------------!
END SUBROUTINE READ_SETTINGS
!**********************************************************************************************************************************!



!**********************************************************************************************************************************!
SUBROUTINE ALLOCATE_THEM(ELENUM,BULKCOMP,SOLCOMP,MASSES,ATOMNO,ELEMENTS,SOLCON,SOLSTRENGTH,YOUNGUNIT,PPTPOPS,PPTSIZE,PPTDENSITY,&
     & PPTSPACING,PPTSHAPE,PPTFACTOR,COHERCON,PPTFRAC,PPTNAME,DELTA,EPSILON,RELEQM,OROWANSTRESS,CUTSTRESS,PPTBETA,PPTDELTA,&
     & PPTEPSILON,PPTKAPPA,PPTBULK,PPTATOMVOL)
IMPLICIT NONE
!----------------------------------------------------------------------------------------------------------------------------------!
INTEGER::I3,J3
INTEGER,INTENT(IN)::ELENUM,PPTPOPS
INTEGER,ALLOCATABLE,DIMENSION(:)::ATOMNO,YOUNGUNIT,PPTSHAPE,RELEQM

DOUBLE PRECISION,ALLOCATABLE,DIMENSION(:,:)::BULKCOMP,SOLCOMP,SOLCON,SOLSTRENGTH
DOUBLE PRECISION,ALLOCATABLE,DIMENSION(:)::MASSES,PPTSIZE,PPTDENSITY,PPTSPACING,PPTFACTOR,PPTFRAC,DELTA,EPSILON,COHERCON
DOUBLE PRECISION,ALLOCATABLE,DIMENSION(:)::OROWANSTRESS,CUTSTRESS,PPTBETA,PPTDELTA,PPTEPSILON,PPTKAPPA,PPTBULK,PPTATOMVOL

CHARACTER(LEN=2),ALLOCATABLE,DIMENSION(:)::ELEMENTS
CHARACTER(LEN=10),ALLOCATABLE,DIMENSION(:)::PPTNAME
!----------------------------------------------------------------------------------------------------------------------------------!
! tell the computer to set aside the appropriate amount of memory to accommodate the arrays for precisely the number of elements.
ALLOCATE(BULKCOMP(4,0:ELENUM),SOLCOMP(4,0:ELENUM),MASSES(0:ELENUM),ELEMENTS(0:ELENUM),SOLCON(4,0:ELENUM),SOLSTRENGTH(3,ELENUM))
ALLOCATE(ATOMNO(0:ELENUM),YOUNGUNIT(0:ELENUM),PPTSHAPE(PPTPOPS),PPTSIZE(PPTPOPS),PPTDENSITY(PPTPOPS),PPTSPACING(PPTPOPS))
ALLOCATE(PPTFACTOR(PPTPOPS),PPTFRAC(PPTPOPS),DELTA(PPTPOPS),EPSILON(PPTPOPS),COHERCON(PPTPOPS),PPTNAME(PPTPOPS),RELEQM(PPTPOPS))
ALLOCATE(OROWANSTRESS(PPTPOPS),CUTSTRESS(PPTPOPS),PPTBETA(PPTPOPS),PPTDELTA(PPTPOPS),PPTEPSILON(PPTPOPS),PPTKAPPA(PPTPOPS))
ALLOCATE(PPTBULK(PPTPOPS),PPTATOMVOL(PPTPOPS))
! initialise the allocated arrrays such that each element ahs a unique value.  If the program executes properly, this has no effect,
! but giving each element a unique value can assist in debugging if needed.
ALLOCINIT:DO I3=0,ELENUM,1
    ATOMNO(I3) = I3+45
    YOUNGUNIT(I3) = -12-I3
    CYCLE_UNITS:DO J3=1,4,1
        BULKCOMP(J3,I3)=-(DBLE(I3)+1.0D+02*DBLE(J3)+9.7663D0)*1.0D+03
        SOLCOMP(J3,I3)=-(DBLE(I3)+1.0D+02*DBLE(J3)+5.6414635D0)*1.0D-04
        SOLCON(J3,I3)=-(DBLE(I3)+1.0D+02*DBLE(J3)+4.468114D+04)
    END DO CYCLE_UNITS
    MASSES(I3)=(DBLE(I3)+8.15351D0)*1.0D+02
    ELEMENTS(I3)='we'
END DO ALLOCINIT
INITPPT:DO I3=1,PPTPOPS,1
    PPTSHAPE(I3) = -50-I3
    PPTSIZE(I3) = 2.644534D+01*DBLE(I3)
    PPTDENSITY(I3) = 4.36647532D+01*DBLE(I3)
    PPTSPACING(I3) = -2.76522D+01*DBLE(I3)
    PPTFACTOR(I3) = -3.243243254D-03*DBLE(I3)
    PPTFRAC(I3) = -2.689469D+02
    DELTA(I3) = -(1.6877537D0+DBLE(I3))*1.0D+03
    EPSILON(I3) = -(8.237662D0+DBLE(I3))*1.0D-02
    PPTNAME(I3) = 'jutrjuuytb'
    COHERCON(I3) = DBLE(I3)*3.0D+00
    OROWANSTRESS(I3) = (4.8694273D0+DBLE(I3))*1.0D+13
    CUTSTRESS(I3) = (2.542D+00*DBLE(I3)+8.2893562856D+00) * 1.0D-05
    PPTBETA(I3) = (DBLE(I3)+5.29237247D+01)
    PPTDELTA(I3) = ( 1.5485495383D+01 + DBLE(I3) ) * 1.0D+04
    PPTEPSILON(I3) = (3.649468946D-01*DBLE(I3)+8.172323D+00)
    PPTKAPPA(I3) = DBLE(I3) + 4.239829D+01 
    PPTBULK(I3) = 4.0D+00*DBLE(I3)
    PPTATOMVOL(I3) = (DBLE(I3) + 3.2308358935D+00) * 1.0D-05
END DO INITPPT

! set the name of the 0th element to be 'Fe' for iron.  The mass/amount of iron will be calculated later.
ELEMENTS(0)='Fe'
!----------------------------------------------------------------------------------------------------------------------------------!
END SUBROUTINE ALLOCATE_THEM
!**********************************************************************************************************************************!



!**********************************************************************************************************************************!
SUBROUTINE READ_PPTS(PPTUNIT,PPTFILE,PPTPOPS,PPTSIZE,PPTDENSITY,PPTFRAC,PPTSPACING,PPTSHAPE,PPTFACTOR,PPTNAME,&
      & RELEQM,PPTBETA,PPTDELTA,PPTEPSILON,PPTKAPPA,PPTBULK,PPTATOMVOL)
IMPLICIT NONE
!----------------------------------------------------------------------------------------------------------------------------------!
INTEGER::I3B,PPTUNIT,HANDLE2
INTEGER,INTENT(IN)::PPTPOPS
INTEGER,DIMENSION(PPTPOPS),INTENT(OUT)::PPTSHAPE,RELEQM

DOUBLE PRECISION,DIMENSION(PPTPOPS),INTENT(OUT)::PPTSIZE,PPTDENSITY,PPTSPACING,PPTFRAC,PPTFACTOR,PPTBETA,PPTDELTA,PPTEPSILON
DOUBLE PRECISION,DIMENSION(PPTPOPS),INTENT(OUT)::PPTKAPPA,PPTBULK,PPTATOMVOL

CHARACTER(LEN=20),INTENT(IN)::PPTFILE
CHARACTER(LEN=10),DIMENSION(PPTPOPS),INTENT(OUT)::PPTNAME
!----------------------------------------------------------------------------------------------------------------------------------!
HANDLE2=0
!----------------------------------------------------------------------------------------------------------------------------------!
OPEN(UNIT=PPTUNIT,IOSTAT=HANDLE2,FILE=PPTFILE,STATUS='OLD',ACTION='READ')
!----------------------------------------------------------------------------------------------------------------------------------!
READ(UNIT=PPTUNIT,FMT=*) ! skip header line
PPTS_READ:DO I3B=1,PPTPOPS,1
    READ(UNIT=PPTUNIT,FMT=*) PPTNAME(I3B),PPTSIZE(I3B),PPTDENSITY(I3B),PPTFRAC(I3B),PPTSPACING(I3B),PPTSHAPE(I3B),&
     & RELEQM(I3B),PPTFACTOR(I3B),PPTBETA(I3B),PPTDELTA(I3B),PPTEPSILON(I3B),PPTKAPPA(I3B),PPTBULK(I3B),PPTATOMVOL(I3B)
    IF(HANDLE2 == -1) THEN
        PRINT 3301,I3B+1,PPTPOPS
        CLOSE(UNIT=PPTUNIT,IOSTAT=HANDLE2,STATUS='KEEP')
        STOP
    ELSE IF(HANDLE2 == -2) THEN
        PRINT 3302,I3B+1
        CLOSE(UNIT=PPTUNIT,IOSTAT=HANDLE2,STATUS='KEEP')
        STOP
    ELSE IF(HANDLE2 .GT. 0) THEN
        PRINT 3303,I3B+1
        CLOSE(UNIT=PPTUNIT,IOSTAT=HANDLE2,STATUS='KEEP')
        STOP
    END IF
END DO PPTS_READ
!----------------------------------------------------------------------------------------------------------------------------------!
CLOSE(UNIT=PPTUNIT,IOSTAT=HANDLE2,STATUS='KEEP')
!----------------------------------------------------------------------------------------------------------------------------------!
3301 FORMAT('There were not enough lines in your precipitate input file.  You provided ',I3,' and you specified ',I3,' types ',&
     & 'of precipitate.  Please change your precipitate input file and try again.  CaNDYFloSS exits safely.')
3302 FORMAT('One or more of the lines in your precipitate configuration file does not have enough values in it.  Please check ',&
     & 'your file against the example file provided and try again.  This error occurred on line ',I3,' of your file.  ',&
     & 'CaNDYFloSS exits safely.')
3303 FORMAT('An error occurred when reading your precipitate configuration file.  Please check that each variable is of the ',&
     & 'correct type (e.g. you have not tried to give a decimal number to a variable that is declared as INTEGER type or have ',&
     & 'not given a letter to something that should only be a number).  There are many possible sources of error and so it is ',&
     & 'not possible to diagnose the problem in advance.  Please check the error code and any other information given in your ',&
     & 'terminal output. CaNDYFloSS exits safely.')
!----------------------------------------------------------------------------------------------------------------------------------!
END SUBROUTINE READ_PPTS
!**********************************************************************************************************************************!



!**********************************************************************************************************************************!
SUBROUTINE READ_COMP(COMPUNIT,COMPFILE,ELENUM,ELEMENTS,BULKCOMP,SOLCOMP,WHATUNIT)
IMPLICIT NONE
!----------------------------------------------------------------------------------------------------------------------------------!
INTEGER::I2,COMPUNIT,HANDLE2
INTEGER,INTENT(IN)::ELENUM
INTEGER,INTENT(IN),DIMENSION(2)::WHATUNIT

DOUBLE PRECISION,DIMENSION(4,0:ELENUM)::BULKCOMP,SOLCOMP

CHARACTER(LEN=2),DIMENSION(0:ELENUM)::ELEMENTS
CHARACTER(LEN=20)::COMPFILE
!----------------------------------------------------------------------------------------------------------------------------------!
! initialise all variables that did not already have values passed to them. The values used are arbitrary and should not affect the
! program - the variables should have other values written to them before they are used for anything.
HANDLE2 = 6546
!----------------------------------------------------------------------------------------------------------------------------------!
! open the file with the name specified in the COMPFILE variable using memory unit SETUNIT.  The 'OLD' status requires that the file
! already exist - if this is not so, the program will fail.  IOSTAT='HANDLE' gives a variable that allows some end-of-file errors to
! be caught and diagnosed, e.g. if some data is missing (0=no errors; <0 = end of record (line) or file reached sooner than
! expected; >0 some other error reading the file, e.g. unrecognised data or format type, such as expecting a number and finding
! letters.).  ACTION='READ' means the file is treated as read-only and is never altered by the program.
OPEN(UNIT=COMPUNIT,IOSTAT=HANDLE2,FILE=COMPFILE,STATUS='OLD',ACTION='READ')
! read in the list of elements and their compositions - note that the amount of iron is not read in, but is calculated as the
! residue of the material.
READ(UNIT=COMPUNIT,FMT=*) ! skip the header line of the composition input file.
COMPREAD:DO I2=1,ELENUM,1
      READ(UNIT=COMPUNIT,FMT=*) ELEMENTS(I2),BULKCOMP(WHATUNIT(1),I2),SOLCOMP(WHATUNIT(1),I2)
! check if the end of file/record variable is showing any errors.
      IF(HANDLE2 > 0) THEN
            PRINT 201
            CLOSE(UNIT=COMPUNIT,IOSTAT=HANDLE2,STATUS='KEEP')
            STOP
      ELSE IF(HANDLE2 == -1) THEN
            PRINT 202
            CLOSE(UNIT=COMPUNIT,IOSTAT=HANDLE2,STATUS='KEEP')
            STOP
      ELSE IF(HANDLE2 == -2) THEN
            PRINT 203,I2+1
            CLOSE(UNIT=COMPUNIT,IOSTAT=HANDLE2,STATUS='KEEP')
            STOP
      END IF
END DO COMPREAD
! close the compositions file.  STATUS='KEEP' means the file will be preserved once closed.
CLOSE(UNIT=COMPUNIT,IOSTAT=HANDLE2,STATUS='KEEP')
! calculate the mass/amount of iron, dependent on whether fractions or percentages were used to express the composition.  Start by
! setting the total amount/mass of solutes to zero and then subtract each of the amounts/masses of each solute entered in the 
! composition file.
IF(WHATUNIT(1) == 1 .OR. WHATUNIT(1) == 2) THEN
      BULKCOMP(WHATUNIT(1),0) = 1.0D+02
      SOLUTE_BULK_PERCENTAGES:DO I2=1,ELENUM,1
            BULKCOMP(WHATUNIT(1),0) = BULKCOMP(WHATUNIT(1),0) - BULKCOMP(WHATUNIT(1),I2)
      END DO SOLUTE_BULK_PERCENTAGES
ELSE IF(WHATUNIT(1) == 3 .OR. WHATUNIT(1) == 4) THEN
      BULKCOMP(WHATUNIT(1),0) = 1.0D+00
      SOLUTE_MATRIX_FRACTIONS:DO I2=1,ELENUM,1
            BULKCOMP(WHATUNIT(1),0) = BULKCOMP(WHATUNIT(1),0) - BULKCOMP(WHATUNIT(1),I2)
      END DO SOLUTE_MATRIX_FRACTIONS
ELSE
      PRINT 204
      STOP
END IF
IF(WHATUNIT(2) == 1 .OR. WHATUNIT(2) == 2) THEN
      SOLCOMP(WHATUNIT(2),0) = 1.0D+02
      SUBTRACT_MATRIX_PERCENTAGES:DO I2=1,ELENUM,1
            SOLCOMP(WHATUNIT(2),0) = SOLCOMP(WHATUNIT(2),0) - SOLCOMP(WHATUNIT(2),I2)
      END DO SUBTRACT_MATRIX_PERCENTAGES
ELSE IF(WHATUNIT(2) == 3 .OR. WHATUNIT(2) == 4) THEN
      SOLCOMP(WHATUNIT(2),0) = 1.0D+00
      SUBTRACT_MATRIX_FRACTIONS:DO I2=1,ELENUM,1
            SOLCOMP(WHATUNIT(2),0) = SOLCOMP(WHATUNIT(2),0) - SOLCOMP(WHATUNIT(2),I2)
      END DO SUBTRACT_MATRIX_FRACTIONS
ELSE
      PRINT 205
      STOP
END IF
!----------------------------------------------------------------------------------------------------------------------------------!
 201  FORMAT('Error reading composition at step ',I2,' (',A2,').  Please check the composition file.  CaNDYFloSS exits safely.')
 202  FORMAT('There were not three data entries in each line.  This error occurred on line ',I2,' of your composition file.  ',&
     & 'CaNDYFloSS exits safely.')
 203  FORMAT('The end of the file was reached before it was expected.  The program reached line ',I2,'of your composition input ',&
     & 'file.  CaNDYFloSS exits safely.')
 204  FORMAT('Unknown type of composition unit of the bulk.  Please check the relevant entry in the configuration file.  ',&
     & 'CaNDYFloSS exits safely.')
 205  FORMAT('Unknown type of composition unit of the solid solution in the matrix.  Please check the relevant entry in the ',&
     & 'configuration file.  CaNDYFloSS exits safely.')
!----------------------------------------------------------------------------------------------------------------------------------!
END SUBROUTINE READ_COMP
!**********************************************************************************************************************************!



!**********************************************************************************************************************************!
SUBROUTINE FIND_MASS(ELENUM,ELEMENTS,RECORDED_ELEMENTS,MASSES,MASSUNIT,MASSFILE)
IMPLICIT NONE
!----------------------------------------------------------------------------------------------------------------------------------!
INTEGER::I4,J4,HANDLE4,DUMMY4A
INTEGER,INTENT(IN)::ELENUM,MASSUNIT

DOUBLE PRECISION,DIMENSION(0:ELENUM)::MASSES
DOUBLE PRECISION,DIMENSION(118)::RECORDED_MASSES

CHARACTER(LEN=12)::DUMMY4B,DUMMY4C
CHARACTER(LEN=2),DIMENSION(0:ELENUM)::ELEMENTS
CHARACTER(LEN=2),DIMENSION(118)::RECORDED_ELEMENTS
CHARACTER(LEN=17),INTENT(IN)::MASSFILE
!----------------------------------------------------------------------------------------------------------------------------------!
! open the file containing the atomic masses and then read in the element symbols, names and masses.
OPEN(UNIT=MASSUNIT,IOSTAT=HANDLE4,FILE=MASSFILE,STATUS='OLD',ACTION='READ')
! read the header line and ignore it
      READ(UNIT=MASSUNIT,FMT=*)
! go through the file containing the atomic masses line-by-line and write the contents to a series of arrays containing the symbol
! for each element, the element names and the element masses. 
      READ_DATA:DO I4=1,118,1
            READ(UNIT=MASSUNIT,FMT=*) DUMMY4A,RECORDED_ELEMENTS(I4),DUMMY4B,RECORDED_MASSES(I4)
      END DO READ_DATA
CLOSE(UNIT=MASSUNIT,IOSTAT=HANDLE4,STATUS='KEEP')
! compare the list of used elements to the list of atomic masses and write the atomic masses for any matched elements to the
! "MASSES" array.
FETCH_MASSES:DO I4 = 0,ELENUM,1
      COMPARE_MASSES:DO J4 = 1,118,1
            IF(ELEMENTS(I4) == RECORDED_ELEMENTS(J4)) THEN
                  MASSES(I4) = RECORDED_MASSES(J4)
                  EXIT COMPARE_MASSES
            ELSE IF(J4 >= 118) THEN
                  PRINT 401
                  STOP
            END IF
      END DO COMPARE_MASSES
END DO FETCH_MASSES
! for vanity, when compiling with NAGFOR, perform some arbitrary calculation with otherwise unreferenced variables to avoid
! warnings
DUMMY4A = 2*DUMMY4A
DUMMY4C=DUMMY4B
DUMMY4B=DUMMY4C
!----------------------------------------------------------------------------------------------------------------------------------!
 401  FORMAT('End of list of atomic masses reached without match.  Please check your element symbols.  CaNDYFloSS exits safely.')
!----------------------------------------------------------------------------------------------------------------------------------!
END SUBROUTINE FIND_MASS
!**********************************************************************************************************************************!



!**********************************************************************************************************************************!
SUBROUTINE CALC_COMPS(ELENUM,ELEMENTS,MASSES,WHATUNIT,BULKCOMP,SOLCOMP)
IMPLICIT NONE
!----------------------------------------------------------------------------------------------------------------------------------!
INTEGER::I5,J5
INTEGER,INTENT(IN)::ELENUM
INTEGER,DIMENSION(2),INTENT(IN)::WHATUNIT

DOUBLE PRECISION::TEMPTOTAL
DOUBLE PRECISION,DIMENSION(0:ELENUM)::TEMPAMOUNT
DOUBLE PRECISION,DIMENSION(0:ELENUM),INTENT(IN)::MASSES
DOUBLE PRECISION,DIMENSION(4,0:ELENUM)::BULKCOMP,SOLCOMP

CHARACTER(LEN=2),DIMENSION(0:ELENUM),INTENT(IN)::ELEMENTS
!----------------------------------------------------------------------------------------------------------------------------------!
! for each possible unit selection, perform a series of calculations to derive the composition expressed in the other three units.
SELECT CASE(WHATUNIT(1))
!----------------------------------------------------------------------------------------------------------------------------------!
      CASE(1)
! reset the total of the intermediate answer to zero.
            TEMPTOTAL = 0.0D+0
! go through each used element in turn and derive the composition in the other three units.
            DO I5=0,ELENUM,1
! divide by 100 to convert mass percentages to mass fractions.
                  BULKCOMP(3,I5) = 1.0D-02 * BULKCOMP(1,I5)
! divide each element's mass percentage by its atomic mass to derive the number of moles per 100kg of alloy.
                  TEMPAMOUNT(I5) = BULKCOMP(1,I5) / MASSES(I5)
! add each number of moles to the total number of moles.
                  TEMPTOTAL = TEMPTOTAL + TEMPAMOUNT(I5)
            END DO
            DO I5=0,ELENUM,1
! divide each element's moles per 100kg of alloy by the total number of moles per 100kg of alloy to derive molar fractions.
                  BULKCOMP(4,I5) = TEMPAMOUNT(I5) / TEMPTOTAL
! repeat the calculation but multiply the results by 100 to derive atomic percentages.
                  BULKCOMP(2,I5) = 1.0D+02 * TEMPAMOUNT(I5) / TEMPTOTAL
            END DO
!----------------------------------------------------------------------------------------------------------------------------------!
      CASE(2)
! reset the total of the intermediate answer to zero.
            TEMPTOTAL = 0.0D+0
! go through each used element in turn and derive the composition in the other three units.
            DO I5=0,ELENUM,1
! divide by 100 to convert atomic percentages to molar fractions.
                  BULKCOMP(4,I5) = 1.0D-02 * BULKCOMP(2,I5)
! multiply each element's atomic percentage by its atomic mass to derive the mass per 100 moles of alloy.
                  TEMPAMOUNT(I5) = BULKCOMP(2,I5) * MASSES(I5)
! add each number of moles to the total mass.
                  TEMPTOTAL = TEMPTOTAL + TEMPAMOUNT(I5)
            END DO
            DO I5=0,ELENUM,1
! divide each element's mass per 100 moles of alloy by the total number of mass per 100 moles of alloy to derive mass fractions.
                  BULKCOMP(3,I5) = TEMPAMOUNT(I5) / TEMPTOTAL
! repeat the calculation but multiply the results by 100 to derive mass percentages.
                  BULKCOMP(1,I5) = 1.0D+02 * TEMPAMOUNT(I5) / TEMPTOTAL
            END DO
!----------------------------------------------------------------------------------------------------------------------------------!
      CASE(3)
! reset the total of the intermediate answer to zero.
            TEMPTOTAL = 0.0D+0
! go through each used element in turn and derive the composition in the other three units.
            DO I5=0,ELENUM,1
! multiply by 100 to convert mass fractions to mass percentages.
                  BULKCOMP(1,I5) = 1.0D+02 * BULKCOMP(3,I5)
! divide each element's mass fraction by its atomic mass to derive the number of moles per kg of alloy.
                  TEMPAMOUNT(I5) = BULKCOMP(3,I5) / MASSES(I5)
! add each number of moles to the total number of moles.
                  TEMPTOTAL = TEMPTOTAL + TEMPAMOUNT(I5)
            END DO
            DO I5=0,ELENUM,1
! divide each element's moles per kg of alloy by the total number of moles per kg of alloy to derive molar fractions.
                  BULKCOMP(4,I5) = TEMPAMOUNT(I5) / TEMPTOTAL
! repeat the calculation but multiply the results by 100 to derive atomic percentages.
                  BULKCOMP(2,I5) = 1.0D+02 * TEMPAMOUNT(I5) / TEMPTOTAL
            END DO
!----------------------------------------------------------------------------------------------------------------------------------!
      CASE(4)
! reset the total of the intermediate answer to zero.
            TEMPTOTAL = 0.0D+0
! go through each used element in turn and derive the composition in the other three units.
            DO I5=0,ELENUM,1
! divide by 100 to convert atomic percentages to molar fractions.
                  BULKCOMP(2,I5) = 1.0D+02 * BULKCOMP(4,I5)
! multiply each element's atomic percentage by its atomic mass to derive the mass per 100 moles of alloy.
                  TEMPAMOUNT(I5) = BULKCOMP(2,I5) * MASSES(I5)
! add each number of moles to the total mass.
                  TEMPTOTAL = TEMPTOTAL + TEMPAMOUNT(I5)
            END DO
            DO I5=0,ELENUM,1
! divide each element's mass per 100 moles of alloy by the total number of mass per 100 moles of alloy to derive mass fractions.
                  BULKCOMP(3,I5) = TEMPAMOUNT(I5) / TEMPTOTAL
! repeat the calculation but multiply the results by 100 to derive mass percentages.
                  BULKCOMP(1,I5) = 1.0D+02 * TEMPAMOUNT(I5) / TEMPTOTAL
            END DO
!----------------------------------------------------------------------------------------------------------------------------------!
END SELECT
!----------------------------------------------------------------------------------------------------------------------------------!
SELECT CASE(WHATUNIT(2))
!----------------------------------------------------------------------------------------------------------------------------------!
      CASE(1)
! reset the total of the intermediate answer to zero.
            TEMPTOTAL = 0.0D+0
! go through each used element in turn and derive the composition in the other three units.
            DO I5=0,ELENUM,1
! divide by 100 to convert mass percentages to mass fractions.
                  SOLCOMP(3,I5) = 1.0D-02 * SOLCOMP(1,I5)
! divide each element's mass percentage by its atomic mass to derive the number of moles per 100kg of alloy.
                  TEMPAMOUNT(I5) = SOLCOMP(1,I5) / MASSES(I5)
! add each number of moles to the total number of moles.
                  TEMPTOTAL = TEMPTOTAL + TEMPAMOUNT(I5)
            END DO
            DO I5=0,ELENUM,1
! divide each element's moles per 100kg of alloy by the total number of moles per 100kg of alloy to derive molar fractions.
                  SOLCOMP(4,I5) = TEMPAMOUNT(I5) / TEMPTOTAL
! repeat the calculation but multiply the results by 100 to derive atomic percentages.
                  SOLCOMP(2,I5) = 1.0D+02 * TEMPAMOUNT(I5) / TEMPTOTAL
            END DO
!----------------------------------------------------------------------------------------------------------------------------------!
      CASE(2)
! reset the total of the intermediate answer to zero.
            TEMPTOTAL = 0.0D+0
! go through each used element in turn and derive the composition in the other three units.
            DO I5=0,ELENUM,1
! divide by 100 to convert atomic percentages to molar fractions.
                  SOLCOMP(4,I5) = 1.0D-02 * SOLCOMP(2,I5)
! multiply each element's atomic percentage by its atomic mass to derive the mass per 100 moles of alloy.
                  TEMPAMOUNT(I5) = SOLCOMP(2,I5) * MASSES(I5)
! add each number of moles to the total mass.
                  TEMPTOTAL = TEMPTOTAL + TEMPAMOUNT(I5)
            END DO
            DO I5=0,ELENUM,1
! divide each element's mass per 100 moles of alloy by the total number of mass per 100 moles of alloy to derive mass fractions.
                  SOLCOMP(3,I5) = TEMPAMOUNT(I5) / TEMPTOTAL
! repeat the calculation but multiply the results by 100 to derive mass percentages.
                  SOLCOMP(1,I5) = 1.0D+02 * TEMPAMOUNT(I5) / TEMPTOTAL
            END DO
!----------------------------------------------------------------------------------------------------------------------------------!
      CASE(3)
! reset the total of the intermediate answer to zero.
            TEMPTOTAL = 0.0D+0
! go through each used element in turn and derive the composition in the other three units.
            DO I5=0,ELENUM,1
! multiply by 100 to convert mass fractions to mass percentages.
                  SOLCOMP(1,I5) = 1.0D+02 * SOLCOMP(3,I5)
! divide each element's mass fraction by its atomic mass to derive the number of moles per kg of alloy.
                  TEMPAMOUNT(I5) = SOLCOMP(3,I5) / MASSES(I5)
! add each number of moles to the total number of moles.
                  TEMPTOTAL = TEMPTOTAL + TEMPAMOUNT(I5)
            END DO
            DO I5=0,ELENUM,1
! divide each element's moles per kg of alloy by the total number of moles per kg of alloy to derive molar fractions.
                  SOLCOMP(4,I5) = TEMPAMOUNT(I5) / TEMPTOTAL
! repeat the calculation but multiply the results by 100 to derive atomic percentages.
                  SOLCOMP(2,I5) = 1.0D+02 * TEMPAMOUNT(I5) / TEMPTOTAL
            END DO
!----------------------------------------------------------------------------------------------------------------------------------!
      CASE(4)
! reset the total of the intermediate answer to zero.
            TEMPTOTAL = 0.0D+0
! go through each used element in turn and derive the composition in the other three units.
            DO I5=0,ELENUM,1
! divide by 100 to convert atomic percentages to molar fractions.
                  SOLCOMP(2,I5) = 1.0D+02 * SOLCOMP(4,I5)
! multiply each element's atomic percentage by its atomic mass to derive the mass per 100 moles of alloy.
                  TEMPAMOUNT(I5) = SOLCOMP(2,I5) * MASSES(I5)
! add each number of moles to the total mass.
                  TEMPTOTAL = TEMPTOTAL + TEMPAMOUNT(I5)
            END DO
            DO I5=0,ELENUM,1
! divide each element's mass per 100 moles of alloy by the total number of mass per 100 moles of alloy to derive mass fractions.
                  SOLCOMP(3,I5) = TEMPAMOUNT(I5) / TEMPTOTAL
! repeat the calculation but multiply the results by 100 to derive mass percentages.
                  SOLCOMP(1,I5) = 1.0D+02 * TEMPAMOUNT(I5) / TEMPTOTAL
            END DO
!----------------------------------------------------------------------------------------------------------------------------------!
END SELECT
!----------------------------------------------------------------------------------------------------------------------------------!
! PRINT the compositions on the screen
PRINT *, ''
PRINT 501
PRINT 502
PRINT 503
PRINT 504
DO I5=0,ELENUM,1
      PRINT 505, ELEMENTS(I5),(SOLCOMP(J5,I5),J5=1,4)
END DO
PRINT 502
PRINT *, ''
!----------------------------------------------------------------------------------------------------------------------------------!
 501  FORMAT('The defined composition is:')
 502  FORMAT(55('-'))
 503  FORMAT('Element |   wt%    |   at.%   | mass frac. | mol. frac.')
 504  FORMAT(8('-'),'|',2(10('-'),'|'),12('-'),'|',11('-'))
 505  FORMAT(3X,A2,3X,2('|',1X,F8.4,1X),2('|',2X,F8.6,2X))
END SUBROUTINE CALC_COMPS
!**********************************************************************************************************************************!



!**********************************************************************************************************************************!
SUBROUTINE READ_COEFFS(COEFILE,COEFUNIT,RECORDED_COEFFS,RECORDED_YOUNGUNIT)
IMPLICIT NONE
!----------------------------------------------------------------------------------------------------------------------------------!
INTEGER::I6,J6,COEFUNIT,HANDLE6
INTEGER,INTENT(OUT),DIMENSION(118)::RECORDED_YOUNGUNIT

DOUBLE PRECISION,DIMENSION(0:3,118),INTENT(OUT)::RECORDED_COEFFS

CHARACTER(LEN=2)::DUMMY6A,DUMMY6B
CHARACTER(LEN=18)::COEFILE
!----------------------------------------------------------------------------------------------------------------------------------!
HANDLE6=24
I6=0
J6=0
!----------------------------------------------------------------------------------------------------------------------------------!
! open the file containing the coefficients for the solid solution strengthening expressions by Young et al., Lelsie and Lacy.
OPEN(UNIT=COEFUNIT,IOSTAT=HANDLE6,FILE=COEFILE,STATUS='OLD',ACTION='READ')
! skip the header line
      READ(UNIT=COEFUNIT,FMT=*)
! go through each element in the periodic table and read in any coefficients
      SOL_COEFFS:DO I6=1,118,1
            READ(UNIT=COEFUNIT,FMT=*) DUMMY6A,(RECORDED_COEFFS(J6,I6),J6=1,3),RECORDED_YOUNGUNIT(I6)
      END DO SOL_COEFFS
      DUMMY6B=DUMMY6A
      DUMMY6A=DUMMY6B
! close the file containing the coefficients for the solid solution strengthening expressions by Young et al., Lelsie and Lacy.
CLOSE(UNIT=COEFUNIT,STATUS='KEEP')
!----------------------------------------------------------------------------------------------------------------------------------!
END SUBROUTINE READ_COEFFS
!**********************************************************************************************************************************!



!**********************************************************************************************************************************!
SUBROUTINE FIND_COEFFS(ELENUM,ELEMENTS,RECORDED_ELEMENTS,RECORDED_COEFFS,SOLCON,ATOMNO,FERTYPE,CNO,RECORDED_YOUNGUNIT,YOUNGUNIT)
IMPLICIT NONE
!----------------------------------------------------------------------------------------------------------------------------------!
INTEGER::I7,J7,K7
INTEGER,INTENT(IN)::ELENUM
INTEGER,INTENT(OUT)::CNO
INTEGER,DIMENSION(0:ELENUM),INTENT(OUT)::ATOMNO
INTEGER,INTENT(IN),DIMENSION(118)::RECORDED_YOUNGUNIT
INTEGER,INTENT(OUT),DIMENSION(0:ELENUM)::YOUNGUNIT

DOUBLE PRECISION,DIMENSION(3,0:ELENUM),INTENT(OUT)::SOLCON
DOUBLE PRECISION,DIMENSION(0:3,118),INTENT(IN)::RECORDED_COEFFS

CHARACTER(LEN=1)::FERTYPE
CHARACTER(LEN=2),DIMENSION(118),INTENT(IN)::RECORDED_ELEMENTS
CHARACTER(LEN=2),DIMENSION(0:ELENUM)::ELEMENTS
CHARACTER(LEN=2),DIMENSION(0:ELENUM)::ELEMENTS2
!----------------------------------------------------------------------------------------------------------------------------------!
! for reasons known only to the FORTRAN Gods, gfortran suddenly started ignoring the final element in the user-specified list when
! the program tried to compare the list to the list of recorded elements.  The list is fine before the do-loop, but contains one
! less entry inside and after the loop.  Copying the list of specified elements to another array of identical size, using this for
! the comparison and then copying it back to the original array solved the problem.  Ideally, the root cause would be found and
! corrected... until then, this hack will have to do!  nagfor suffers no such issues, however.
REPLICATE:DO I7=0,ELENUM,1
    ELEMENTS2(I7) = ELEMENTS(I7)
END DO REPLICATE

! compare te user-specified list of elements to the standard list and copy the necessary solid solution strengthening coefficients
! and a variable to indicate whether the expression compiled from  Young and Bhadeshia, Nagakai et al. and Takeuchi et al. depends
! on the mass% or mole%.
PICK_COEFFS:DO I7=0,ELENUM,1
      COMPARE_ELEMENT:DO J7=1,118,1
            IF(ELEMENTS2(I7) == RECORDED_ELEMENTS(J7)) THEN
! record what atomic number (i.e. row number in RECORDED_ELEMENTS) each element is: this information will be used later.
                  ATOMNO(I7) = J7
                  YOUNGUNIT(I7) = RECORDED_YOUNGUNIT(J7)
                  COPY_COEFFS:DO K7=1,3,1
                        SOLCON(K7,I7) = RECORDED_COEFFS(K7,J7)
                  END DO COPY_COEFFS
            END IF
      END DO COMPARE_ELEMENT

END DO PICK_COEFFS
DO I7=1,ELENUM,1
    ELEMENTS(I7) = ELEMENTS2(I7)
END DO
!----------------------------------------------------------------------------------------------------------------------------------!
! find the position in the list occupied by carbon.  Calculate the contribution of carbon and use the (now known) position to put
! the contribution in the appropriate place in the array.
FINDC:DO I7=0,ELENUM,1
    IF(ELEMENTS(I7) == 'C') THEN
! if the ferritic phase is identified as (diffusional) ferrite or bainite, use the expression derived by Speich and Warlimont.
        IF(FERTYPE == 'F' .OR. FERTYPE == 'B') THEN
            SOLCON(1,I7) = 1.7255D+03
! the carbon contribution is identical irrespective of the model used for substitutional solutes, so copy it twice.
            SOLCON(2,I7) = SOLCON(1,I7)
            SOLCON(3,I7) = SOLCON(1,I7)
! if the ferritic phase is identified as martensite, use the alternative expression derived by Winchell and Cohen.
        ELSE IF(FERTYPE == 'M') THEN
            SOLCON(1,I7) = 1.1713D+03
! the carbon contribution is identical irrespective of the model used for substitutional solutes, so copy it twice.
            SOLCON(2,I7) = SOLCON(1,I7)
            SOLCON(3,I7) = SOLCON(1,I7)
        ELSE
            PRINT 701
            STOP
        END IF
        CNO = I7
        EXIT FINDC
    END IF
! if this is the last time through the loop and `no match is found, PRINT a message to that effect and move on.
    IF(I7 == ELENUM) THEN
        PRINT 702
    END IF
END DO FINDC
!----------------------------------------------------------------------------------------------------------------------------------!
 701  FORMAT('Type of ferrite not understood.  Please check your input file to ensure ferrite is set to one of "M", "B", or "F".',&
    & '  CaNDYFloSS exits safely.')
 702  FORMAT('Carbon not found in the composition input file.  Please double-check the file and ensure that carbon is either "C" ',&
     & 'or "C ", i.e. with no leading spaces.  CaNDYFloSS will proceed with no carbon strengthening.')
!----------------------------------------------------------------------------------------------------------------------------------!
END SUBROUTINE FIND_COEFFS
!**********************************************************************************************************************************!



!**********************************************************************************************************************************!
SUBROUTINE SOLSOLSTR(ELENUM,ELEMENTS,SOLCON,SOLCOMP,SOLSTRENGTH,TOTSOLSTR,FERTYPE,CNO,YOUNGUNIT)
IMPLICIT NONE
!----------------------------------------------------------------------------------------------------------------------------------!
INTEGER::I8,J8
INTEGER,INTENT(IN)::ELENUM,CNO
INTEGER,INTENT(IN),DIMENSION(0:ELENUM)::YOUNGUNIT

DOUBLE PRECISION,DIMENSION(3,0:ELENUM),INTENT(IN)::SOLCON
DOUBLE PRECISION,DIMENSION(3,0:ELENUM),INTENT(OUT)::SOLSTRENGTH
DOUBLE PRECISION,DIMENSION(4,0:ELENUM),INTENT(IN)::SOLCOMP
DOUBLE PRECISION,DIMENSION(3)::TOTSOLSTR

CHARACTER(LEN=1)::FERTYPE
CHARACTER(LEN=2),DIMENSION(0:ELENUM),INTENT(IN)::ELEMENTS
!----------------------------------------------------------------------------------------------------------------------------------!
! reset all the contributions to solid solution strengthening and the total solid solution strengthening from each expression.
RESET:DO I8=1,3,1
            DO J8=1,ELENUM,1
                  SOLSTRENGTH(I8,J8) = 0.0D+00
            END DO
            TOTSOLSTR(I8) = 0.0D+00
END DO RESET
!----------------------------------------------------------------------------------------------------------------------------------!
! calculate the solid solution strengthening using models by (1) Young & Bhadeshia, Takeuchi, Takaki, Narito and others; (2) Lacy;
! (3) Leslie.
!----------------------------------------------------------------------------------------------------------------------------------!
PRINT 810
PRINT 811
PRINT 810
PRINT 812
PRINT 813
YOUNGSOL:DO I8=1,ELENUM,1
    IF(I8 == CNO) THEN
        IF(FERTYPE == 'M') THEN
            SOLSTRENGTH(1,I8) = SOLCOMP(1,I8) ** (1.0D+03 / 3.0D+03) * SOLCON(2,I8)
        ELSE IF(FERTYPE == 'B' .OR. FERTYPE == 'F') THEN
            SOLSTRENGTH(1,I8) = SOLCOMP(1,I8) ** (5.0D-01) * SOLCON(2,I8)        
        ELSE
            PRINT 801
        END IF
        TOTSOLSTR(1) = TOTSOLSTR(1) + SOLSTRENGTH(1,I8)
        PRINT 814, ELEMENTS(I8),SOLCOMP(1,I8),SOLCON(1,I8),SOLSTRENGTH(1,I8),TOTSOLSTR(1),'mass'
        CYCLE YOUNGSOL
    END IF
    IF(ABS(SOLCON(1,I8)) .GE. 1.0D-03) THEN
! for each element, check the list of atomic numbers of used elements to determine if mass percentage or atomic percentage should
! be used in the calculation of yield stress.
        IF(YOUNGUNIT(I8) == 1) THEN
! the case for coefficients expressed in terms of wt%
            SOLSTRENGTH(1,I8) = SOLCOMP(1,I8) * SOLCON(1,I8)
            TOTSOLSTR(1) = TOTSOLSTR(1) + SOLSTRENGTH(1,I8)
            PRINT 814, ELEMENTS(I8),SOLCOMP(1,I8),SOLCON(1,I8),SOLSTRENGTH(1,I8),TOTSOLSTR(1),'mass'
        ELSE IF(YOUNGUNIT(I8) == 2) THEN
! the case for coefficients expressed in terms of at.%
            SOLSTRENGTH(1,I8) = SOLCOMP(2,I8) * SOLCON(1,I8)
            TOTSOLSTR(1) = TOTSOLSTR(1) + SOLSTRENGTH(1,I8)
            PRINT 814, ELEMENTS(I8),SOLCOMP(2,I8),SOLCON(1,I8),SOLSTRENGTH(1,I8),TOTSOLSTR(1),'mole'
        ELSE
            PRINT 809
            STOP
        END IF
    ELSE
        SOLSTRENGTH(1,I8) = 0.0D+00
        TOTSOLSTR(1) = TOTSOLSTR(1) + SOLSTRENGTH(1,I8)
        IF(YOUNGUNIT(I8) == 1) THEN
            PRINT 814, ELEMENTS(I8),SOLCOMP(1,I8),SOLCON(1,I8),SOLSTRENGTH(1,I8),TOTSOLSTR(1),'mass'
        ELSE IF(YOUNGUNIT(I8) == 2) THEN
            PRINT 814, ELEMENTS(I8),SOLCOMP(1,I8),SOLCON(1,I8),SOLSTRENGTH(1,I8),TOTSOLSTR(1),'mole'
        ELSE
            PRINT 814, ELEMENTS(I8),SOLCOMP(1,I8),SOLCON(1,I8),SOLSTRENGTH(1,I8),TOTSOLSTR(1),'none'
        END IF
    END IF

END DO YOUNGSOL

PRINT 810
PRINT 815, TOTSOLSTR(1)
PRINT 810
PRINT *,''
!----------------------------------------------------------------------------------------------------------------------------------!
PRINT 802
PRINT 808
PRINT 802
PRINT 805
PRINT 804
LACY:DO I8=0,ELENUM,1
    IF(I8 == CNO) THEN
        IF(FERTYPE == 'M') THEN
            SOLSTRENGTH(2,I8) = SOLCOMP(1,I8) ** (1.0D+03 / 3.0D+03) * SOLCON(2,I8)
        ELSE IF(FERTYPE == 'B' .OR. FERTYPE == 'F') THEN
            SOLSTRENGTH(2,I8) = SOLCOMP(1,I8) ** (5.0D-01) * SOLCON(2,I8)        
        ELSE
            PRINT 801
        END IF
        TOTSOLSTR(2) = TOTSOLSTR(2) + SOLSTRENGTH(2,I8)
        PRINT 806, ELEMENTS(I8),SOLCOMP(1,I8),SOLCON(2,I8),SOLSTRENGTH(2,I8),TOTSOLSTR(2)
    ELSE
        SOLSTRENGTH(2,I8) = SOLCOMP(2,I8) * SOLCON(2,I8)
        TOTSOLSTR(2) = TOTSOLSTR(2) + SOLSTRENGTH(2,I8)
        PRINT 806, ELEMENTS(I8),SOLCOMP(2,I8),SOLCON(2,I8),SOLSTRENGTH(2,I8),TOTSOLSTR(2)
    END IF
END DO LACY
PRINT 802
PRINT 807, TOTSOLSTR(2)
PRINT 802
PRINT *, ''
!----------------------------------------------------------------------------------------------------------------------------------!
PRINT 802
PRINT 803
PRINT 802
PRINT 805
PRINT 804
LESLIE:DO I8=0,ELENUM,1
    IF(I8 == CNO) THEN
        IF(FERTYPE == 'M') THEN
            SOLSTRENGTH(3,I8) = SOLCOMP(1,I8) ** (1.0D+03 / 3.0D+03) * SOLCON(3,I8)
        ELSE IF(FERTYPE == 'B' .OR. FERTYPE == 'F') THEN
            SOLSTRENGTH(3,I8) = SOLCOMP(1,I8) ** (5.0D-01) * SOLCON(3,I8)        
        ELSE
            PRINT 801
        END IF
        TOTSOLSTR(3) = TOTSOLSTR(3) + SOLSTRENGTH(3,I8)
        PRINT 806, ELEMENTS(I8),SOLCOMP(2,I8),SOLCON(3,I8),SOLSTRENGTH(3,I8),TOTSOLSTR(3)
    ELSE
        SOLSTRENGTH(3,I8) = SOLCOMP(2,I8) * SOLCON(3,I8)
        TOTSOLSTR(3) = TOTSOLSTR(3) + SOLSTRENGTH(3,I8)
        PRINT 806, ELEMENTS(I8),SOLCOMP(2,I8),SOLCON(3,I8),SOLSTRENGTH(3,I8),TOTSOLSTR(3)
    END IF
END DO LESLIE
PRINT 802
PRINT 807, TOTSOLSTR(3)
PRINT 802
PRINT *, ''
!----------------------------------------------------------------------------------------------------------------------------------!
 801  FORMAT('Type of ferrite not understood.  Please check your input file to ensure ferrite is set to one of "M", "B", or "F".',&
    & '  CanDYFLoSS exits safely.')
 802  FORMAT(127('-'))
 803  FORMAT(27X,'Substitutional solid solution strengthening according to model by Leslie',28X)
 804  FORMAT(9('-'),'|',10('-'),'|',38('-'),'|',34('-'),'|',32('-'))
 805  FORMAT(' Element |   at.%   | Strengthening coefficient / MPa/at.% | Strengthening contribution / MPa | ', &
     & 'Cumulative strengthening / MPa')
 806  FORMAT(3X,A2,4X,'|',1X,F8.5,1X,'|',11X,F12.5,15X,'|',11X,F12.5,11X,'|',10X,F12.5,10X,I2)
 807  FORMAT(59X,'Total substitutional solid solution stregnthening = ',F12.5,' MPa')
 808  FORMAT(28X,'Substitutional solid solution strengthening according to model by Lacy',29X)
 809  FORMAT('Unrecognised unit type for the substitutional solid solution strengthening model by Young & Bhadeshia and Takeuchi ',&
    & '& Takaki.  CaNDYFloSS exits safely.')
 810  FORMAT(138('-'))
 811  FORMAT(10X,'Substitutional solid solution strengthening according to model by Young, Bhadeshia, Narita et al. and ', &
     & 'Takeuchi et al.')
 812  FORMAT(' Element |  amount  |   Strengthening coefficient / MPa/%  | Strengthening contribution / MPa | ', &
     & 'Cumulative strengthening / MPa | Unit (%)')
 813  FORMAT(9('-'),'|',10('-'),'|',38('-'),'|',34('-'),'|',32('-'),'|',10('-'))
 814  FORMAT(3X,A2,4X,'|',1X,F8.5,1X,'|',11X,F12.5,15X,'|',11X,F12.5,11X,'|',10X,F12.5,10X,'|',3X,A4)
 815  FORMAT(67X,'Total substitutional solid solution stregnthening = ',F12.5,' MPa')
!----------------------------------------------------------------------------------------------------------------------------------!
END SUBROUTINE SOLSOLSTR
!**********************************************************************************************************************************!



!**********************************************************************************************************************************!
SUBROUTINE COHERENCY(PPTPOPS,PPTSIZE,PPTDENSITY,PPTSPACING,PPTSHAPE,PPTFACTOR,COHERCON,PPTFRAC,DELTA,EPSILON,YOUNGMOD,BULKMOD,&
     & POISSON,MATRIXSPACING,RELEQM,PPTNAME)
IMPLICIT NONE
!----------------------------------------------------------------------------------------------------------------------------------!
INTEGER::I9
INTEGER,INTENT(IN)::PPTPOPS
INTEGER,DIMENSION(PPTPOPS),INTENT(IN)::PPTSHAPE,RELEQM
! reminder - intent(in) means variable cannot be altered within this subroutine; intent(out) means that the value is wiped upon
! entering the subroutine and MUST be set somewhere within the subroutine; intent(inout), equivalent to no intent(...) means the
! incoming value is retained but can be altered within the subroutine.  The latest is used for variables passed to the subroutine
! that are EITHER specified by the user in the input files OR are calculated on-the-fly.
DOUBLE PRECISION,INTENT(IN)::YOUNGMOD,BULKMOD,POISSON,MATRIXSPACING
DOUBLE PRECISION,DIMENSION(PPTPOPS),INTENT(IN)::PPTDENSITY,PPTSPACING,PPTSIZE
DOUBLE PRECISION,DIMENSION(PPTPOPS),INTENT(INOUT)::PPTFACTOR,PPTFRAC
DOUBLE PRECISION,DIMENSION(PPTPOPS),INTENT(OUT)::DELTA,EPSILON,COHERCON

CHARACTER(LEN=10),DIMENSION(PPTPOPS),INTENT(IN)::PPTNAME
!----------------------------------------------------------------------------------------------------------------------------------!
MOTTCALC:DO I9=1,PPTPOPS,1
! calculate the mismatch in interatomic spacings between matrix and precipitate, relative to the spacing in the matrix.
    IF(RELEQM(I9) == 1) THEN
! the following expression applies if the lattice parameter of the precipitate is known, and so any relaxation has already occurred.
! If those data are unavailable, the expression should include the equilibrium interatomic spacing for the precipitate and is then
! the expression for delta in Mott and Nabarro's 1940 paper.
        EPSILON(I9) = PPTSPACING(I9) / MATRIXSPACING - 1.0D+00
    ELSE
! the following is theoretical value for epsilon and may be implemented if the measured value is unavailable and the above
! expression is delta. 
        DELTA(I9) =  PPTSPACING(I9) / MATRIXSPACING - 1.0D+00
        EPSILON(I9) = ( 3 * BULKMOD * DELTA(I9) ) / (3 * BULKMOD * ( 1 + POISSON ) + 2 * YOUNGMOD )
    END IF
! calculate the appropriate shape factor (to convert the cube of 'size' (radius for spherical particles and half side length for
! cubic precipitates)) to volume of a single precipitate. 
        IF(PPTSHAPE(I9) == 1) THEN
            PPTFACTOR(I9) = 4.18879020478638D+00 ! 4*pi/3
        ELSE IF(PPTSHAPE(I9) == 2) THEN
            PPTFACTOR(I9) = 8.0D+00
        END IF
        IF(PPTFRAC(I9) .LT. 0.0D+00) THEN
! calculate the volume fraction of precipitates if they are not already known... otherwise use the values specified by the user.
            PPTFRAC(I9) = PPTDENSITY(I9) * PPTFACTOR(I9) * PPTSIZE(I9)**3.0D+00
        END IF
! calculate the strengthening contribution from each precipitate population.  Take the magnitude of epsilon only, as compressive and
! tensile stress fields will behave similarly.
        COHERCON(I9) = YOUNGMOD * ABS(EPSILON(I9)) * PPTFRAC(I9)
        PRINT 901, 1.0D-06*COHERCON(I9),PPTNAME(I9)
END DO MOTTCALC
PRINT *,''
!----------------------------------------------------------------------------------------------------------------------------------!
 901  FORMAT('Strength contribution due to coherency strains is ',F5.1,' MPa due to ',A10)
!----------------------------------------------------------------------------------------------------------------------------------!
END SUBROUTINE COHERENCY
!**********************************************************************************************************************************!



!**********************************************************************************************************************************!
SUBROUTINE OROWAN(PPTPOPS,PPTSIZE,PPTDENSITY,PPTNAME,MATRIXSPACING,POISSON,SHEARMOD,TAYLOR,OROWANSTRESS)
IMPLICIT NONE
!----------------------------------------------------------------------------------------------------------------------------------!
INTEGER::I10
INTEGER,INTENT(IN)::PPTPOPS

DOUBLE PRECISION::BURGERS
DOUBLE PRECISION,INTENT(IN)::MATRIXSPACING,SHEARMOD,POISSON,TAYLOR
DOUBLE PRECISION,DIMENSION(PPTPOPS),INTENT(OUT)::OROWANSTRESS
DOUBLE PRECISION,DIMENSION(PPTPOPS),INTENT(IN)::PPTSIZE,PPTDENSITY

CHARACTER(LEN=10),DIMENSION(PPTPOPS),INTENT(IN)::PPTNAME
!----------------------------------------------------------------------------------------------------------------------------------!
! convert the matrix lattice parameter to the Burgers vector.  For ferrite, this involves dividing by the square root of 3.  Convert
! from Angstroms to metres so it is compatible with other lengths in the following expression.
BURGERS = 1.0D-10 * MATRIXSPACING * ((3.0D+00) ** (-5.0D-01))
OROWANLOOP:DO I10=1,PPTPOPS,1
print *, i10,poisson,taylor,shearmod,burgers,pptdensity(i10),pptsize(i10)
! calculate Orowan bowing stress.  0.15915... is 1/(2pi).  Use the cube root of the number density to give approximate
! interprecipitate spacings.  Subtract twice the precipitate radius to change from spacinge between centres to spacing in the bulk.
   OROWANSTRESS(I10) = 1.591549430919D-01 * ((1.0D+00-POISSON)**(-0.5D+00)) * TAYLOR * SHEARMOD * BURGERS * &
     & (PPTDENSITY(I10) ** (-3.333333333333D-01) - PPTSIZE(I10))**(-1.0D+00) * LOG(PPTSIZE(I10) / BURGERS)
    PRINT 1001, 1.0D-06*OROWANSTRESS(I10),PPTNAME(I10)
END DO OROWANLOOP
PRINT *, ''
!----------------------------------------------------------------------------------------------------------------------------------!
 1001 FORMAT('The Orowan bowing stress is ',E10.1,' MPa. for ',A10)
!----------------------------------------------------------------------------------------------------------------------------------!
END SUBROUTINE OROWAN
!**********************************************************************************************************************************!



!**********************************************************************************************************************************!
SUBROUTINE CUTTING(PPTPOPS,PPTFRAC,PPTSIZE,TAYLOR,CUTSTRESS,SHEARMOD,PPTBULK,MATRIXSPACING,PPTBETA,&
     & PPTDELTA,PPTEPSILON,PPTKAPPA,MATRIXATOMVOL,PPTATOMVOL,YOUNGMOD,POISSON,PPTNAME)
IMPLICIT NONE
!----------------------------------------------------------------------------------------------------------------------------------!
INTEGER::I11,PPTPOPS

DOUBLE PRECISION::BURGERS
DOUBLE PRECISION,INTENT(IN)::TAYLOR,SHEARMOD,MATRIXSPACING,MATRIXATOMVOL,YOUNGMOD,POISSON
DOUBLE PRECISION,DIMENSION(PPTPOPS)::PPTBETA,PPTDELTA,PPTEPSILON,PPTKAPPA,PPTFRAC
DOUBLE PRECISION,DIMENSION(PPTPOPS),INTENT(IN)::PPTSIZE,PPTBULK,PPTATOMVOL
DOUBLE PRECISION,DIMENSION(PPTPOPS),INTENT(OUT)::CUTSTRESS

CHARACTER(LEN=10),DIMENSION(PPTPOPS),INTENT(IN)::PPTNAME
!----------------------------------------------------------------------------------------------------------------------------------!
! for each precipitate type, test whether the beta variable is provided or should be calculated.  The constant with 6.598... is a 
! theoretical constant equal to 0.5(3^1.5+4^1.5), according to E. A. Wilson Mater. Sci. Technol. 14 (1998) p277-282.
BURGERS = MATRIXSPACING * (3.0D+00 ** (-5.0D-01))
CUTCALC:DO I11=1,PPTPOPS,1
    IF(PPTBETA(I11) .LT. 0.0D+00) THEN
! allow another value of PPTKAPPA to be provided by the user, otherwise use the theoretical value.
        IF(PPTKAPPA(I11) .LT. 0.0D+00) THEN
            PPTKAPPA(I11) = 6.59807621135332D+00
        END IF
! allow the effect of materials properties to be accounted for, either from literature (provided from file), or based on
! metallurgical properties.
        IF(PPTEPSILON(I11) .LT. 1.0D+02) then
! calculate the linear strain associated by precipitation - 1 subtracted from the cube root of the ratio of atomic volumes.
            IF(PPTDELTA(I11) .LT. -1.0D+02) THEN
                PPTDELTA(I11) = (PPTATOMVOL(I11) / MATRIXATOMVOL) ** (3.33333333333D-01) - 1.0D+00
            END IF
            PPTEPSILON(I11) = 3.0D+00*PPTBULK(I11)*(1.0D+00+POISSON)*PPTDELTA(I11) / (3.0D+00*PPTBULK(I11)*(1+POISSON)+ &
     & 2.0D+00*YOUNGMOD)
        END IF
! calculate the constant of precipitate cutting to be used to determine the contribution to yield stress.
! The constant 0.477... is 3/(2pi)
        PPTBETA(I11) = (PPTKAPPA(I11) * PPTEPSILON(I11)) ** (1.5D+00) * SHEARMOD * 4.7746482927568D-01 * (BURGERS**(-1.0D-10))
    END IF
! the precipitate volume fraction are either read in or have already been calculated in the subroutine "COHERENCY".
    CUTSTRESS(I11) = TAYLOR * PPTBETA(I11) * PPTFRAC(I11)**(5.0D-01) * PPTSIZE(I11)**(5.0D-01)
    PRINT 1101, 1.0D-06*CUTSTRESS(I11),PPTNAME(I11)
END DO CUTCALC
PRINT *, ''
!----------------------------------------------------------------------------------------------------------------------------------!
 1101 FORMAT('The strengthening due to cutting of precipitates is ',F6.1,' MPa for ',A10)
!----------------------------------------------------------------------------------------------------------------------------------!
END SUBROUTINE CUTTING
!**********************************************************************************************************************************!



!**********************************************************************************************************************************!
SUBROUTINE WORKHARDENING(SHEARMOD,FERTYPE,INITGRAINSIZE,MATRIXSPACING,STRAINSTEP,PATHCONST,ENDSTRAIN,TAYLOR,HARDCONST,&
    & RECOVERYCONST,STRAINRATE,TEMPERATURE,SHEARTEMP,GB_ENERGY,MOBILITY,ZENER_FACTOR,YIELD,RESULTSFILE,DEFACT,REXACT,REXCONST,&
    & REXRHO,GROWTHACT)
IMPLICIT NONE
!----------------------------------------------------------------------------------------------------------------------------------!
INTEGER::I12,DIDITREX,WRITERR
INTEGER,PARAMETER::RESUNIT=18

DOUBLE PRECISION,INTENT(IN)::SHEARMOD,INITGRAINSIZE,STRAINSTEP,PATHCONST,TAYLOR,HARDCONST,RECOVERYCONST,ENDSTRAIN,MATRIXSPACING
DOUBLE PRECISION,INTENT(IN)::TEMPERATURE,SHEARTEMP,STRAINRATE,GB_ENERGY,ZENER_FACTOR,DEFACT,REXACT,REXCONST,REXRHO,GROWTHACT
DOUBLE PRECISION::RHO_INIT,FREEPATH,HARDENING,STRAIN,RHO,BURGERS,RHO_CRIT,ZENER,GRAINSIZE,THE_MOBILITY
DOUBLE PRECISION::PEAKSTRAIN,CRITSTRAIN,THALF,STARSTRAIN,REXGRAIN,REXFRAC,TIMESTEP
DOUBLE PRECISION,PARAMETER::GASCONST=8.31446D+00
DOUBLE PRECISION,DIMENSION(2),INTENT(IN)::MOBILITY
DOUBLE PRECISION,DIMENSION(3),INTENT(IN)::YIELD

CHARACTER(LEN=1),INTENT(IN)::FERTYPE
CHARACTER(LEN=20),INTENT(IN)::RESULTSFILE
!----------------------------------------------------------------------------------------------------------------------------------!
IF(FERTYPE == 'F') THEN
! taken from Bainite in Steels.
    RHO_INIT = 1.0D+14
ELSE IF(FERTYPE == 'M' .OR. FERTYPE == 'B') THEN
! apply the model from Bainite in Steels, refitted using gnuplot to these coefficients.
    RHO_INIT =  1.0D+01 ** (1.1866D+01 + (3.77394D+03/(SHEARTEMP+2.7315D+02)) - (8.999728903D+05/((SHEARTEMP+2.7315D+02)**2.0D+00)))
ELSE
    PRINT 1201
    RHO_INIT = 1.0D+14
END IF

THE_MOBILITY = MOBILITY(1)*EXP(-MOBILITY(2)/(GASCONST*(TEMPERATURE+2.7315D+02)))
RHO = RHO_INIT
RHO_CRIT = 1.0D+99

ZENER = STRAINRATE * EXP(DEFACT/(GASCONST*(TEMPERATURE+2.7315D+02)))
GRAINSIZE = 1.0D+06 * INITGRAINSIZE
PEAKSTRAIN = 6.97D-04 * GRAINSIZE**(3.0D-01) * ZENER**(1.7D-01)
 CRITSTRAIN = 8.0D-01 * PEAKSTRAIN
BURGERS = (1.0D-10) * MATRIXSPACING * (5.0D-01) * (3.0D0**(5.0D-01))
STARSTRAIN = 5.7D-01 * INITGRAINSIZE**(1.7D-01) * PEAKSTRAIN
TIMESTEP = STRAINSTEP / STRAINRATE

STRAIN = 0.0D+00
REXFRAC = 0.0D+00
THALF = 1.0D+32
WRITERR = 0

! print headers or the results table on the screen.
PRINT 1202
PRINT 1203
PRINT 1204
PRINT 1202

! print headers for the results file.
OPEN(UNIT=RESUNIT,FILE=RESULTSFILE,STATUS='REPLACE') ! this line DELETES any file in the same directory/folder with the same name.
WRITE(UNIT=RESUNIT,FMT=1208)
WRITE(UNIT=RESUNIT,FMT=1209)
CLOSE(UNIT=RESUNIT,STATUS='KEEP')

! set the flag that indicates if recrystallisation occurred to zero.  It will be changed to 1 if recrystallisation takes place.
DIDITREX = 0

STRAINIT:DO
! test to see if the strain has exceeded the required level.  If so, exit the loop, if not, do another strain step.
    IF(STRAIN .GT. ENDSTRAIN) THEN
        EXIT STRAINIT
    END IF
! calculate the mean free path for dislocations.
    FREEPATH = ( GRAINSIZE**(-1.0D+00) + PATHCONST*(RHO)**(5.0D-01) )**(-1.0D+00)
! calculate the contribution to flow stress.
    HARDENING = HARDCONST * TAYLOR * SHEARMOD * BURGERS * (RHO**(5.0D-01))
! calculate the lower-bound (intragranular) critical dislocation density for recrytallisation, as derived by Roberts and Ahlblom,
! Acta Metall. 26 (1977) p. 801-813, equation 4.
    RHO_CRIT = (1.6D+01*GB_ENERGY*STRAINRATE/(3.0D+00*BURGERS*((5.0D-01*SHEARMOD*BURGERS**(2.0D+00))**(2.0D+00))*FREEPATH* & 
    & THE_MOBILITY))**(1.0D+00/3.0D+00)
! print the current values of strain, dislocation density, effective work hardening at room temperature, effective work hardening at
! the deformation temperature and overall yield stress considering each of the three solis solution strengthening models to the
! screen.
    PRINT 1205, STRAIN,RHO,(1.0D-06*HARDENING),(ZENER_FACTOR*1.0D-06*HARDENING),&
    & (YIELD(I12)+(ZENER_FACTOR*1.0D-06*HARDENING),I12=1,3)
! open the results file
    OPEN(UNIT=RESUNIT,FILE=RESULTSFILE,STATUS='OLD',POSITION='APPEND',ACTION='WRITE')
! write the current data to the results file.
    WRITE(UNIT=RESUNIT,FMT=1210,ADVANCE='YES',IOSTAT=WRITERR) STRAIN,RHO,(1.0D-06*HARDENING),&
    & (ZENER_FACTOR*1.0D-06*HARDENING),(YIELD(I12)+(ZENER_FACTOR*1.0D-06*HARDENING),I12=1,3)
! decide whether the write was OK and quit if not.
    IF(WRITERR /= 0) THEN
        PRINT 1211
        CLOSE(UNIT=RESUNIT,STATUS='KEEP')
        STOP
    END IF
! close the results file.  Although repeated opening and closing slows the program execution, it does offer protection in case the
! program crashes - if the program terminates with the file open , data may be lost.
    CLOSE(UNIT=RESUNIT,STATUS='KEEP')
! calculate the dislocation density after an increment of strain - equation 8 in Bouaziz & Guelton with the identity for free path
! substituted in for the term in brackets.
    RHO = RHO + (STRAINSTEP*TAYLOR) * ( (BURGERS*FREEPATH)**(-1.0D+00) - RECOVERYCONST*RHO )

    STRAIN = STRAIN + STRAINSTEP

! calculate the time required to recrystallise 50% of the material given the current conditions.  The expression differs if the
! critical strain for recrystallisation has been reached and has three alternative expressions, depending on the temperature if it
! has not.
    IF(STRAIN .GT. CRITSTRAIN) THEN
        THALF = 1.06D+00 * ZENER**(-6.0D+00) * EXP(REXACT/(GASCONST*(TEMPERATURE+2.7315D+00)))
    ELSE IF(TEMPERATURE .GE. 1.004D+03) THEN
        THALF = 2.5D-19 * (1.0D+06*INITGRAINSIZE)**(2.0D+00) * STRAIN**(-4.0D+00) * &
    & EXP(3.25D+05/(GASCONST*(TEMPERATURE+2.7315D+00)))
    ELSE IF(TEMPERATURE .GE. 8.91D+02) THEN
        THALF = 5.94D-38 * (1.0D+06*INITGRAINSIZE)**(2.0D+00) * STRAIN**(-4.0D+00) * &
    & EXP(7.8D+05/(GASCONST*(TEMPERATURE+2.7315D+00)))
    ELSE
        THALF = 9.24D-09 * (1.0D+06*INITGRAINSIZE)**(2.0D+00) * STRAIN**(-4.0D+00) * &
    & EXP(1.3D+05/(GASCONST*(TEMPERATURE+2.7315D+00)))
    END IF
! calculate what fraction of the material would recrystallise under the current conditions if recrystallisation is possible.
    REXFRAC = 1.0D+00 - EXP(-6.93D-01*(TIMESTEP/THALF)**REXCONST)
! if the strain exceeds that required to begin recrystallisation, add all the effects in to the situation, otherwise leave things
! unchanged.
    IF(STRAIN .GE. CRITSTRAIN) THEN
! change the flag to indicate that recrystallisation occurs.
        DIDITREX = 1
! calculate the size of a newly recrystallised grain, according to Roberts and Ahlblom.
        REXGRAIN = ((6.0D+00*BURGERS*FREEPATH*THE_MOBILITY*(5.0D-01*SHEARMOD*BURGERS**2.0D+00))/(5.0D+00*STRAINRATE))*RHO_CRIT
! allow the newly formed grains to grow throughout half the timestep, giving an average grain size.
        REXGRAIN = (REXGRAIN**(1.0D+01) + 3.87D+32 * (5.0D-01*TIMESTEP) * EXP(-GROWTHACT/(GASCONST*TEMPERATURE)))**(1.0D-01)
! calculate a new average dislocation density.
        RHO = RHO*(1.0D+00 - REXFRAC) + REXRHO*REXFRAC
! calculate a new average grain size.
        GRAINSIZE = GRAINSIZE*(1.0D+00 - REXFRAC) + REXGRAIN*REXFRAC
    END IF

END DO STRAINIT

PRINT 1202

! print a statement confirming whether or not recrystallisation was allowed in the program.
IF(DIDITREX==0) THEN
    PRINT 1206
ELSE
    PRINT 1207
END IF
PRINT *, ''
!----------------------------------------------------------------------------------------------------------------------------------!
 1201 FORMAT('Ferrite type ',A1,' not recognised.  The dislocation density before strain is set to a reasonable value for a ',&
    &'heavily annealed material, namely 10^9 m^(-2).')
 1202 FORMAT(133('-'))
 1203 FORMAT('  Strain   | Disln density / m^(-2) |  RT Strengthening / MPa | Strengthening / MPa |',15X,'Yield stress / MPa')
 1204 FORMAT(11X,'|',24X,'|',25X,'|',21X,'|',4X,'Lacy',4X,'|',3X,'Leslie',3X,'|',1X,'Young/Takeuchi et al.')
 1205 FORMAT(E10.3,' | ',6X,E10.3,6X,' | ',7X,F6.1,11X,'|',7X,F6.1,8X,2('|',3X,F6.1,3X),'|',6X,F6.1)
 1206 FORMAT(8('*'),'The calculated strain did not exceed that required to initiate recrystallisation at any time during the ',&
    & 'deformation.',9('*'))
 1207 FORMAT(14('*'),1X,'The calculated dislocation density exceeded that required for recrystallisation during the deformation.',&
    & 15('*'))
 1208 FORMAT('Strain,Dislocation density / m^(-2),RT Strengthening / MPa,Strengthening / MPa,Yield stress / MPa,,')
 1209 FORMAT(',,,,Lacy,Leslie et al.,"Young & Bhadeshia, Takeuchi et al., etc."')
 1210 FORMAT(2(E10.3,','),4(F6.1,','),F6.1)
 1211 FORMAT('Error writing to results file.  CaNDYFloSS exits safely.  No previous data were lost due to the error.')
!----------------------------------------------------------------------------------------------------------------------------------!
END SUBROUTINE WORKHARDENING
!**********************************************************************************************************************************!



!**********************************************************************************************************************************!
SUBROUTINE ADDITALL(PPTPOPS,CUTSTRESS,OROWANSTRESS,COHERCON,PPTNAME,PEIERLS,SIZESTRENGTH,TOTSOLSTR,YIELDMODELNAME,YIELD)
IMPLICIT NONE
!----------------------------------------------------------------------------------------------------------------------------------!
INTEGER::I13,J13
INTEGER,INTENT(IN)::PPTPOPS

DOUBLE PRECISION,INTENT(IN)::PEIERLS,SIZESTRENGTH
DOUBLE PRECISION,DIMENSION(3),INTENT(IN)::COHERCON,TOTSOLSTR
DOUBLE PRECISION,DIMENSION(3),INTENT(OUT)::YIELD
DOUBLE PRECISION,INTENT(IN),DIMENSION(PPTPOPS)::CUTSTRESS,OROWANSTRESS

CHARACTER(LEN=6),DIMENSION(3),INTENT(IN)::YIELDMODELNAME
CHARACTER(LEN=10),DIMENSION(PPTPOPS),INTENT(IN)::PPTNAME
!----------------------------------------------------------------------------------------------------------------------------------!
! for each precipitate, decide whether cutting or bowing dominates (whichever is lower) and add it to the total.
ADD_PPTS:DO I13=1,PPTPOPS,1
    IF(CUTSTRESS(I13) .GT. OROWANSTRESS(I13)) THEN
        PRINT 1301, PPTNAME(I13)
        DO J13=1,3,1
            YIELD(J13) = 1.0D-06*OROWANSTRESS(I13) + 1.0D-06*COHERCON(I13)
        END DO
    ELSE IF(OROWANSTRESS(I13) .GT. CUTSTRESS(I13)) THEN
        PRINT 1302, PPTNAME(I13)
        DO J13=1,3,1
            YIELD(J13) = 1.0D-06*CUTSTRESS(I13) + 1.0D-06*COHERCON(I13)
        END DO
    ELSE
        PRINT 1303, PPTNAME(I13)
        DO J13=1,3,1
            YIELD(J13) = 1.0D-06*OROWANSTRESS(I13) + 1.0D-06*COHERCON(I13)
        END DO
    END IF
END DO ADD_PPTS

PRINT *, ''

PRINTITALL:DO I13=1,3,1
    YIELD(I13) = YIELD(I13) + PEIERLS + SIZESTRENGTH + TOTSOLSTR(I13)
    PRINT 1304, YIELDMODELNAME(I13),YIELD(I13),NINT(3.0D-01*YIELD(I13))
END DO PRINTITALL
PRINT *, ''
!**********************************************************************************************************************************!
 1301  FORMAT('For ',A10,' bowing occurs before cutting.  The Orowan bowing stress is therefore taken as the precipitate ',&
    & 'strengthening mechanism.')
 1302  FORMAT('For ',A10,' cutting occurs before bowing.  The cutting stress is therefore taken as the precipitate strengthening ',&
    & 'mechanism.')
 1303  FORMAT('For ',A10,' both cutting and bowing occur simultaneously.  That stress is therefore taken as the precipitate ',&
    & 'strengthening mechanism.')
 1304  FORMAT('Total yield strength including the substitutional solid solution strengthening according to the model by ',A6,&
    & ' is ',F8.1,' MPa.  Estiated hardness is ',I3,' HV.')
!**********************************************************************************************************************************!
END SUBROUTINE ADDITALL
!**********************************************************************************************************************************!



!**********************************************************************************************************************************!
SUBROUTINE YIELDTOFLOW(YIELD,FLOW,FLOW_TEMP_AIM,STR_RATE_AIM,ACTIVATION,FERTYPE,EXPONENT,STR_RATE_STATIC,AMBIENT,ZENER_FACTOR)
IMPLICIT NONE
!----------------------------------------------------------------------------------------------------------------------------------!
INTEGER::I14

DOUBLE PRECISION,INTENT(IN)::FLOW_TEMP_AIM,STR_RATE_AIM,EXPONENT,STR_RATE_STATIC,AMBIENT
DOUBLE PRECISION,INTENT(INOUT)::ACTIVATION,ZENER_FACTOR
DOUBLE PRECISION,DIMENSION(3),INTENT(IN)::YIELD
DOUBLE PRECISION,DIMENSION(3),INTENT(OUT)::FLOW

CHARACTER(LEN=1),INTENT(IN)::FERTYPE
!----------------------------------------------------------------------------------------------------------------------------------!
! determine the correct activation energy for dislocation motion.  This can be user-specified (if the value given is >0) or assigned
! based on Zener and Holloman, J. App. Phys. 15 (1944), p.22-32.  Should further values of Q be determined or found for other 
! situations, they may be added inside the following if loop.
IF(ACTIVATION .LT. 0.0D+00) THEN
    IF(FERTYPE == 'F') THEN
        PRINT 1401
! this is 10,000 calories, found by Zener and Holloman for pearlite and probably most applicable for diffusional ferrite.
        ACTIVATION = 4.184D+04
    ELSE IF(FERTYPE == 'M' .OR. FERTYPE == 'B') THEN
        PRINT 1402
! this is 12,000 calories, found by Zener and Holloman for cast medium-carbon steels (written in figure 7; confirmed using
! regression of 0.01% stress line in figure 13), so probably suitable for martensite/bainite.
        ACTIVATION = 5.0208D+04
! Zener and Holloman also found the value of the activation energy to be 14,000 calories (58,576 J) for forged medium carbon steel.
! Linear regression of Fig. 7 was used to derive the constant of proportionality between 0.1% proof stress and strain rate for
! forged medium carbon steels as 16.4 MPa s.
    ELSE
! in this case, the "FERTYPE" variable is not legal, so ignore temperature changes
        PRINT 1403
    END IF
ELSE
! this branch is reached if a non-negative value of activation energy is specified.  This is then used in the calculations.
    PRINT 1404, 1.0D-03*ACTIVATION
END IF
PRINT *, ''
! convert the static, ambient-temperature yield stresses to flow stresse at a specified strain rate and temperature.
! use the expression derived by Zener and Holloman, J. App. Phys. 15 (1944) p.23-33.
ZENER_FACTOR = (STR_RATE_AIM/STR_RATE_STATIC)**EXPONENT * &
    & EXP(ACTIVATION * EXPONENT * ( FLOW_TEMP_AIM**(-1.0D+00) - AMBIENT**(-1.0D+00) ) / 8.3145D+00)
! apply the conversion factor to the three initial yield stresses calculated thus far.
DO I14=1,3,1
    FLOW(I14) = YIELD(I14) * ZENER_FACTOR
    PRINT 1405, FLOW_TEMP_AIM,STR_RATE_AIM,YIELD(I14),FLOW(I14)
END DO
PRINT *, ''
!----------------------------------------------------------------------------------------------------------------------------------!
 1401  FORMAT('Activation energy for dislocation motion set to 41,840J in accordance with the value for pearlite (and used for ',&
    & 'diffusional ferrite) in Zener and Holloman, J. App. Phys. 15 (1944), p.22-32.')
 1402  FORMAT('Activation energy for dislocation motion set to 41,840J in accordance with the value for cast, medium-carbon steel',&
    & ' in Zener and Holloman, J. App. Phys. 15 (1944), p.22-32.')
 1403  FORMAT('The type of ferrite is not recognised.  Please check that you have correctly specified M = martensite, B = bainite',&
    & ' of F = (diffusonal) ferrite in your config.dat file.  CaNDYFloSS will continue with no temperature effects.')
 1404  FORMAT('User-specified value of activation energy used: ',F6.1,' kJ')
 1405  FORMAT('Flow stress at ',F5.1,' C and ',E7.1,' / s with static ambient yield stress of ',F6.1,' MPa is ',F6.1,' MPa.')
!----------------------------------------------------------------------------------------------------------------------------------!
END SUBROUTINE YIELDTOFLOW
!**********************************************************************************************************************************!



!**********************************************************************************************************************************!
! calculate the effect of grain refinement on the yield stress.
PURE DOUBLE PRECISION FUNCTION REFINEMENT(GRAINSIZE)
IMPLICIT NONE
!----------------------------------------------------------------------------------------------------------------------------------!
DOUBLE PRECISION,INTENT(IN)::GRAINSIZE
!----------------------------------------------------------------------------------------------------------------------------------!
! check whether the grain size is 1um or larger, or less than one micron.
IF(GRAINSIZE .GE. 1.0D-06) THEN
! for grains larger than 1um, use the model by Hall and Petch, where the strengthening is inversely proportional to the square
! root of the grain size.
    REFINEMENT = 6.0D+02 * (1.0D+06*GRAINSIZE) ** (-5.0D-01)
ELSE
! for grains less than 1um in size, use the model by Langford and Cohen that the strengthening is 115 / (grain size in metres) Pa.
    REFINEMENT = 1.15D-04 * (1.0D+06*GRAINSIZE) ** (-1.0D+00)
END IF

END FUNCTION REFINEMENT
!**********************************************************************************************************************************!
